@@ -18,6 +18,7 @@
 #endif
 
 #include <stdio.h>
+#include <strings.h>
 
 void cmdline_print_message(FILE* target, const char* str)
 {
@@ -38,6 +39,8 @@ void cmdline_cancel_computations(bool &delayed, bool &immediately)
 {
 }
 
+using namespace shogun;
+
 const int32_t READLINE_BUFFER_SIZE = 10000;
 extern CSGInterface* interface;
 extern CSGInterfaceMethod sg_methods[];
@@ -62,19 +65,21 @@ void CCmdLineInterface::reset(const char* line)
 		return;
 
 	char* element=NULL;
-	char delim_equal[]="=";
-	char delim_lhs[]="=, \t\n";
-	char delim_rhs[]=" \t\n";
+	const char delim_equal[]="=";
+	const char delim_lhs[]="=, \t\n";
+	const char delim_rhs[]=" \t\n";
 
 	SG_UNREF(m_lhs);
 	m_lhs=NULL;
 	SG_UNREF(m_rhs);
 	m_rhs=NULL;
 
-	// split lhs from rhs
-	char* equal_sign=strstr(line, delim_equal);
+	/* split lhs from rhs
+	 * for some reason strstr on sunos and newer libc's
+	 * requires a char* haystack
+	 */
+	char* equal_sign=strstr((char*) line, delim_equal);
 	if (equal_sign)
-	//if (strstr(line, delim_equal))
 	{
 #ifdef DEBUG_CMDLINEIF
 		SG_PRINT("has lhs!\n");
