@@ -10,6 +10,8 @@
 
 #include "classifier/LinearClassifier.h"
 
+using namespace shogun;
+
 CLinearClassifier::CLinearClassifier()
 : CClassifier(), w_dim(0), w(NULL), bias(0), features(NULL)
 {
@@ -31,7 +33,7 @@ bool CLinearClassifier::save(FILE* dstfile)
 	return false;
 }
 
-CLabels* CLinearClassifier::classify(CLabels* output)
+CLabels* CLinearClassifier::classify()
 {
 	if (features)
 	{
@@ -42,8 +44,7 @@ CLabels* CLinearClassifier::classify(CLabels* output)
 		float64_t* out=new float64_t[num];
 		features->dense_dot_range(out, 0, num, NULL, w, w_dim, bias);
 
-		if (!output)
-			output=new CLabels(num);
+		CLabels* output=new CLabels(num);
 		output->set_labels(out, num);
 
 		delete[] out;
@@ -53,3 +54,13 @@ CLabels* CLinearClassifier::classify(CLabels* output)
 
 	return NULL;
 }
+
+CLabels* CLinearClassifier::classify(CFeatures* data)
+{
+	if (!data)
+		SG_ERROR("No features specified\n");
+	if (!data->has_property(FP_DOT))
+		SG_ERROR("Specified features are not of type CDotFeatures\n");
+	set_features((CDotFeatures*) data);
+	return classify();
+}
