@@ -15,14 +15,21 @@
 #include "classifier/svm/SVM.h"
 #include "classifier/svm/SVM_libsvm.h"
 
-#include <stdio.h>
+namespace shogun
+{
+enum LIBSVM_SOLVER_TYPE
+{
+	LIBSVM_C_SVC = 1,
+	LIBSVM_NU_SVC = 2
+};
 
 /** @brief LibSVM */
 class CLibSVM : public CSVM
 {
 	public:
 		/** constructor */
-		CLibSVM();
+		CLibSVM(LIBSVM_SOLVER_TYPE st=LIBSVM_C_SVC);
+
 		/** constructor
 		 *
 		 * @param C constant C
@@ -33,8 +40,15 @@ class CLibSVM : public CSVM
 
 		virtual ~CLibSVM();
 
-		/** train SVM */
-		virtual bool train();
+		/** train SVM classifier
+		 *
+		 * @param data training data (parameter can be avoided if distance or
+		 * kernel-based classifiers are used and distance/kernels are
+		 * initialized with train data)
+		 *
+		 * @return whether training was successful
+		 */
+		virtual bool train(CFeatures* data=NULL);
 
 		/** get classifier type
 		 *
@@ -44,13 +58,17 @@ class CLibSVM : public CSVM
 
 		/** @return object name */
 		inline virtual const char* get_name() const { return "LibSVM"; }
+
 	protected:
 		/** SVM problem */
 		svm_problem problem;
 		/** SVM param */
 		svm_parameter param;
-
 		/** SVM model */
 		struct svm_model* model;
+
+		/** solver type */
+		LIBSVM_SOLVER_TYPE solver_type;
 };
+}
 #endif
