@@ -1,23 +1,23 @@
 /*
  * Copyright (c) 2000-2009 Chih-Chung Chang and Chih-Jen Lin
  * All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * 1. Redistributions of source code must retain the above copyright
  * notice, this list of conditions and the following disclaimer.
- * 
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  * notice, this list of conditions and the following disclaimer in the
  * documentation and/or other materials provided with the distribution.
- * 
+ *
  * 3. Neither name of copyright holders nor the names of its contributors
  * may be used to endorse or promote products derived from this software
  * without specific prior written permission.
- * 
- * 
+ *
+ *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -39,6 +39,7 @@
 #include "classifier/svm/SVM_libsvm.h"
 #include "kernel/Kernel.h"
 #include "lib/io.h"
+#include "lib/Signal.h"
 #include "lib/common.h"
 #include "lib/Mathematics.h"
 
@@ -48,15 +49,13 @@
 #include <float.h>
 #include <string.h>
 #include <stdarg.h>
+
+namespace shogun
+{
+
 typedef KERNELCACHE_ELEM Qfloat;
 typedef float64_t schar;
-#ifndef min
-template <class T> inline T min(T x,T y) { return (x<y)?x:y; }
-#endif
-#ifndef max
-template <class T> inline T max(T x,T y) { return (x>y)?x:y; }
-#endif
-template <class T> inline void swap(T& x, T& y) { T t=x; x=y; y=t; }
+
 template <class S, class T> inline void clone(T*& dst, S* src, int32_t n)
 {
 	dst = new T[n];
@@ -66,6 +65,9 @@ template <class S, class T> inline void clone(T*& dst, S* src, int32_t n)
 #define TAU 1e-12
 #define Malloc(type,n) (type *)malloc((n)*sizeof(type))
 
+class QMatrix;
+class SVC_QMC;
+
 //
 // Kernel Cache
 //
@@ -105,7 +107,7 @@ Cache::Cache(int32_t l_, int64_t size_):l(l_),size(size_)
 	head = (head_t *)calloc(l,sizeof(head_t));	// initialized to 0
 	size /= sizeof(Qfloat);
 	size -= l * sizeof(head_t) / sizeof(Qfloat);
-	size = max(size, (int64_t) 2*l);	// cache must be large enough for two columns
+	size = CMath::max(size, (int64_t) 2*l);	// cache must be large enough for two columns
 	lru_head.next = lru_head.prev = &lru_head;
 }
 
@@ -154,7 +156,7 @@ int32_t Cache::get_data(const int32_t index, Qfloat **data, int32_t len)
 		// allocate new space
 		h->data = (Qfloat *)realloc(h->data,sizeof(Qfloat)*len);
 		size -= more;
-		swap(h->len,len);
+		CMath::swap(h->len,len);
 	}
 
 	lru_insert(h);
@@ -168,18 +170,18 @@ void Cache::swap_index(int32_t i, int32_t j)
 
 	if(head[i].len) lru_delete(&head[i]);
 	if(head[j].len) lru_delete(&head[j]);
-	swap(head[i].data,head[j].data);
-	swap(head[i].len,head[j].len);
+	CMath::swap(head[i].data,head[j].data);
+	CMath::swap(head[i].len,head[j].len);
 	if(head[i].len) lru_insert(&head[i]);
 	if(head[j].len) lru_insert(&head[j]);
 
-	if(i>j) swap(i,j);
+	if(i>j) CMath::swap(i,j);
 	for(head_t *h = lru_head.next; h!=&lru_head; h=h->next)
 	{
 		if(h->len > i)
 		{
 			if(h->len > j)
-				swap(h->data[i],h->data[j]);
+				CMath::swap(h->data[i],h->data[j]);
 			else
 			{
 				// give up
@@ -217,8 +219,8 @@ public:
 	virtual Qfloat *get_QD() const = 0;
 	virtual void swap_index(int32_t i, int32_t j) const	// no so const...
 	{
-		swap(x[i],x[j]);
-		if(x_square) swap(x_square[i],x_square[j]);
+		CMath::swap(x[i],x[j]);
+		if(x_square) CMath::swap(x_square[i],x_square[j]);
 	}
 
 	inline float64_t kernel_function(int32_t i, int32_t j) const
@@ -327,19 +329,19 @@ protected:
 	virtual float64_t calculate_rho();
 	virtual void do_shrinking();
 private:
-	bool be_shrunk(int32_t i, float64_t Gmax1, float64_t Gmax2);	
+	bool be_shrunk(int32_t i, float64_t Gmax1, float64_t Gmax2);
 };
 
 void Solver::swap_index(int32_t i, int32_t j)
 {
 	Q->swap_index(i,j);
-	swap(y[i],y[j]);
-	swap(G[i],G[j]);
-	swap(alpha_status[i],alpha_status[j]);
-	swap(alpha[i],alpha[j]);
-	swap(p[i],p[j]);
-	swap(active_set[i],active_set[j]);
-	swap(G_bar[i],G_bar[j]);
+	CMath::swap(y[i],y[j]);
+	CMath::swap(G[i],G[j]);
+	CMath::swap(alpha_status[i],alpha_status[j]);
+	CMath::swap(alpha[i],alpha[j]);
+	CMath::swap(p[i],p[j]);
+	CMath::swap(active_set[i],active_set[j]);
+	CMath::swap(G_bar[i],G_bar[j]);
 }
 
 void Solver::reconstruct_gradient()
@@ -357,7 +359,7 @@ void Solver::reconstruct_gradient()
 	for(j=0;j<active_size;j++)
 		if(is_free(j))
 			nr_free++;
-	
+
 	if (nr_free*l > 2*active_size*(l-active_size))
 	{
 		for(i=active_size;i<l;i++)
@@ -413,6 +415,7 @@ void Solver::Solve(
 	}
 
 	// initialize gradient
+	CSignal::clear_cancel();
 	{
 		G = new float64_t[l];
 		G_bar = new float64_t[l];
@@ -422,7 +425,10 @@ void Solver::Solve(
 			G[i] = p_p[i];
 			G_bar[i] = 0;
 		}
-		for(i=0;i<l;i++)
+		SG_SINFO("Computing gradient for initial set of non-zero alphas\n");
+		//CMath::display_vector(alpha, l, "alphas");
+		for(i=0;i<l && !CSignal::cancel_computations();i++)
+		{
 			if(!is_lower_bound(i))
 			{
 				const Qfloat *Q_i = Q->get_Q(i,l);
@@ -434,20 +440,23 @@ void Solver::Solve(
 					for(j=0;j<l;j++)
 						G_bar[j] += get_C(i) * Q_i[j];
 			}
+			SG_SPROGRESS(i, 0, l);
+		}
+		SG_SDONE();
 	}
 
 	// optimization step
 
 	int32_t iter = 0;
-	int32_t counter = min(l,1000)+1;
+	int32_t counter = CMath::min(l,1000)+1;
 
-	while(1)
+	while (!CSignal::cancel_computations())
 	{
 		// show progress and do shrinking
 
 		if(--counter == 0)
 		{
-			counter = min(l,1000);
+			counter = CMath::min(l,1000);
 			if(shrinking) do_shrinking();
 			//SG_SINFO(".");
 		}
@@ -468,11 +477,11 @@ void Solver::Solve(
 		}
 
 		SG_SABS_PROGRESS(gap, -CMath::log10(gap), -CMath::log10(1), -CMath::log10(eps), 6);
-		
+
 		++iter;
 
 		// update alpha[i] and alpha[j], handle bounds carefully
-		
+
 		const Qfloat *Q_i = Q->get_Q(i,active_size);
 		const Qfloat *Q_j = Q->get_Q(j,active_size);
 
@@ -491,7 +500,7 @@ void Solver::Solve(
 			float64_t diff = alpha[i] - alpha[j];
 			alpha[i] += delta;
 			alpha[j] += delta;
-			
+
 			if(diff > 0)
 			{
 				if(alpha[j] < 0)
@@ -573,7 +582,7 @@ void Solver::Solve(
 
 		float64_t delta_alpha_i = alpha[i] - old_alpha_i;
 		float64_t delta_alpha_j = alpha[j] - old_alpha_j;
-		
+
 		for(int32_t k=0;k<active_size;k++)
 		{
 			G[k] += Q_i[k]*delta_alpha_i + Q_j[k]*delta_alpha_j;
@@ -609,6 +618,21 @@ void Solver::Solve(
 						G_bar[k] += C_j * Q_j[k];
 			}
 		}
+
+#ifdef MCSVM_DEBUG
+		// calculate objective value
+		{
+			float64_t v = 0;
+			for(i=0;i<l;i++)
+				v += alpha[i] * (G[i] + p[i]);
+
+			p_si->obj = v/2;
+
+			float64_t primal=0;
+			//float64_t gap=100000;
+			SG_SPRINT("dual obj=%f primal obf=%f gap=%f\n", v/2, primal, gap);
+		}
+#endif
 	}
 
 	// calculate rho
@@ -654,7 +678,7 @@ int32_t Solver::select_working_set(
 	// j: minimizes the decrease of obj value
 	//    (if quadratic coefficient <= 0, replace it with tau)
 	//    -y_j*grad(f)_j < -y_i*grad(f)_i, j in I_low(\alpha)
-	
+
 	float64_t Gmax = -INF;
 	float64_t Gmax2 = -INF;
 	int32_t Gmax_idx = -1;
@@ -697,7 +721,7 @@ int32_t Solver::select_working_set(
 					Gmax2 = G[j];
 				if (grad_diff > 0)
 				{
-					float64_t obj_diff; 
+					float64_t obj_diff;
 					float64_t quad_coef=Q_i[i]+QD[j]-2.0*y[i]*Q_i[j];
 					if (quad_coef > 0)
 						obj_diff = -(grad_diff*grad_diff)/quad_coef;
@@ -721,7 +745,7 @@ int32_t Solver::select_working_set(
 					Gmax2 = -G[j];
 				if (grad_diff > 0)
 				{
-					float64_t obj_diff; 
+					float64_t obj_diff;
 					float64_t quad_coef=Q_i[i]+QD[j]+2.0*y[i]*Q_i[j];
 					if (quad_coef > 0)
 						obj_diff = -(grad_diff*grad_diff)/quad_coef;
@@ -777,27 +801,27 @@ void Solver::do_shrinking()
 	// find maximal violating pair first
 	for(i=0;i<active_size;i++)
 	{
-		if(y[i]==+1)	
+		if(y[i]==+1)
 		{
-			if(!is_upper_bound(i))	
+			if(!is_upper_bound(i))
 			{
 				if(-G[i] >= Gmax1)
 					Gmax1 = -G[i];
 			}
-			if(!is_lower_bound(i))	
+			if(!is_lower_bound(i))
 			{
 				if(G[i] >= Gmax2)
 					Gmax2 = G[i];
 			}
 		}
-		else	
+		else
 		{
-			if(!is_upper_bound(i))	
+			if(!is_upper_bound(i))
 			{
 				if(-G[i] >= Gmax2)
 					Gmax2 = -G[i];
 			}
-			if(!is_lower_bound(i))	
+			if(!is_lower_bound(i))
 			{
 				if(G[i] >= Gmax1)
 					Gmax1 = G[i];
@@ -805,7 +829,7 @@ void Solver::do_shrinking()
 		}
 	}
 
-	if(unshrink == false && Gmax1 + Gmax2 <= eps*10) 
+	if(unshrink == false && Gmax1 + Gmax2 <= eps*10)
 	{
 		unshrink = true;
 		reconstruct_gradient();
@@ -840,16 +864,16 @@ float64_t Solver::calculate_rho()
 		if(is_upper_bound(i))
 		{
 			if(y[i]==-1)
-				ub = min(ub,yG);
+				ub = CMath::min(ub,yG);
 			else
-				lb = max(lb,yG);
+				lb = CMath::max(lb,yG);
 		}
 		else if(is_lower_bound(i))
 		{
 			if(y[i]==+1)
-				ub = min(ub,yG);
+				ub = CMath::min(ub,yG);
 			else
-				lb = max(lb,yG);
+				lb = CMath::max(lb,yG);
 		}
 		else
 		{
@@ -866,6 +890,35 @@ float64_t Solver::calculate_rho()
 	return r;
 }
 
+
+//
+//Solve with individually weighted examples
+//
+class WeightedSolver : public Solver
+{
+
+public:
+
+	WeightedSolver(float64_t* cost_vec)
+	{
+
+		this->Cs = cost_vec;
+
+	}
+
+	virtual float64_t get_C(int32_t i)
+	{
+
+		return Cs[i];
+	}
+
+protected:
+
+  float64_t* Cs;
+
+};
+
+
 //
 // Solver for nu-svm classification and regression
 //
@@ -881,7 +934,7 @@ public:
 		float64_t p_eps, SolutionInfo* p_si, int32_t shrinking)
 	{
 		this->si = p_si;
-		Solver::Solve(p_l,p_Q,p,p_y,p_alpha,p_Cp,p_Cn,p_eps,p_si,shrinking);
+		Solver::Solve(p_l,p_Q,p_p,p_y,p_alpha,p_Cp,p_Cn,p_eps,p_si,shrinking);
 	}
 private:
 	SolutionInfo *si;
@@ -947,14 +1000,14 @@ int32_t Solver_NU::select_working_set(
 	{
 		if(y[j]==+1)
 		{
-			if (!is_lower_bound(j))	
+			if (!is_lower_bound(j))
 			{
 				float64_t grad_diff=Gmaxp+G[j];
 				if (G[j] >= Gmaxp2)
 					Gmaxp2 = G[j];
 				if (grad_diff > 0)
 				{
-					float64_t obj_diff; 
+					float64_t obj_diff;
 					float64_t quad_coef = Q_ip[ip]+QD[j]-2*Q_ip[j];
 					if (quad_coef > 0)
 						obj_diff = -(grad_diff*grad_diff)/quad_coef;
@@ -978,7 +1031,7 @@ int32_t Solver_NU::select_working_set(
 					Gmaxn2 = -G[j];
 				if (grad_diff > 0)
 				{
-					float64_t obj_diff; 
+					float64_t obj_diff;
 					float64_t quad_coef = Q_in[in]+QD[j]-2*Q_in[j];
 					if (quad_coef > 0)
 						obj_diff = -(grad_diff*grad_diff)/quad_coef;
@@ -995,7 +1048,7 @@ int32_t Solver_NU::select_working_set(
 		}
 	}
 
-	gap=max(Gmaxp+Gmaxp2,Gmaxn+Gmaxn2);
+	gap=CMath::max(Gmaxp+Gmaxp2,Gmaxn+Gmaxn2);
 	if(gap < eps)
 		return 1;
 
@@ -1016,14 +1069,14 @@ bool Solver_NU::be_shrunk(
 	{
 		if(y[i]==+1)
 			return(-G[i] > Gmax1);
-		else	
+		else
 			return(-G[i] > Gmax4);
 	}
 	else if(is_lower_bound(i))
 	{
 		if(y[i]==+1)
 			return(G[i] > Gmax2);
-		else	
+		else
 			return(G[i] > Gmax3);
 	}
 	else
@@ -1052,14 +1105,14 @@ void Solver_NU::do_shrinking()
 		if(!is_lower_bound(i))
 		{
 			if(y[i]==+1)
-			{	
+			{
 				if(G[i] > Gmax2) Gmax2 = G[i];
 			}
 			else	if(G[i] > Gmax3) Gmax3 = G[i];
 		}
 	}
 
-	if(unshrink == false && max(Gmax1+Gmax2,Gmax3+Gmax4) <= eps*10) 
+	if(unshrink == false && CMath::max(Gmax1+Gmax2,Gmax3+Gmax4) <= eps*10)
 	{
 		unshrink = true;
 		reconstruct_gradient();
@@ -1094,9 +1147,9 @@ float64_t Solver_NU::calculate_rho()
 		if(y[i]==+1)
 		{
 			if(is_upper_bound(i))
-				lb1 = max(lb1,G[i]);
+				lb1 = CMath::max(lb1,G[i]);
 			else if(is_lower_bound(i))
-				ub1 = min(ub1,G[i]);
+				ub1 = CMath::min(ub1,G[i]);
 			else
 			{
 				++nr_free1;
@@ -1106,9 +1159,9 @@ float64_t Solver_NU::calculate_rho()
 		else
 		{
 			if(is_upper_bound(i))
-				lb2 = max(lb2,G[i]);
+				lb2 = CMath::max(lb2,G[i]);
 			else if(is_lower_bound(i))
-				ub2 = min(ub2,G[i]);
+				ub2 = CMath::min(ub2,G[i]);
 			else
 			{
 				++nr_free2;
@@ -1122,21 +1175,398 @@ float64_t Solver_NU::calculate_rho()
 		r1 = sum_free1/nr_free1;
 	else
 		r1 = (ub1+lb1)/2;
-	
+
 	if(nr_free2 > 0)
 		r2 = sum_free2/nr_free2;
 	else
 		r2 = (ub2+lb2)/2;
-	
+
 	si->r = (r1+r2)/2;
 	return (r1-r2)/2;
 }
 
+class SVC_QMC: public LibSVMKernel
+{
+public:
+	SVC_QMC(const svm_problem& prob, const svm_parameter& param, const schar *y_, int32_t n_class, float64_t fac)
+	:LibSVMKernel(prob.l, prob.x, param)
+	{
+		nr_class=n_class;
+		factor=fac;
+		clone(y,y_,prob.l);
+		cache = new Cache(prob.l,(int64_t)(param.cache_size*(1l<<20)));
+		QD = new Qfloat[prob.l];
+		for(int32_t i=0;i<prob.l;i++)
+		{
+			QD[i]= factor*(nr_class-1)*kernel_function(i,i);
+		}
+	}
+
+	Qfloat *get_Q(int32_t i, int32_t len) const
+	{
+		Qfloat *data;
+		int32_t start;
+		if((start = cache->get_data(i,&data,len)) < len)
+		{
+			for(int32_t j=start;j<len;j++)
+			{
+				if (y[i]==y[j])
+					data[j] = factor*(nr_class-1)*kernel_function(i,j);
+				else
+					data[j] = -factor*kernel_function(i,j);
+			}
+		}
+		return data;
+	}
+
+	inline Qfloat get_orig_Qij(Qfloat Q, int32_t i, int32_t j)
+	{
+		if (y[i]==y[j])
+			return Q/(nr_class-1);
+		else
+			return -Q;
+	}
+
+	Qfloat *get_QD() const
+	{
+		return QD;
+	}
+
+	void swap_index(int32_t i, int32_t j) const
+	{
+		cache->swap_index(i,j);
+		LibSVMKernel::swap_index(i,j);
+		CMath::swap(y[i],y[j]);
+		CMath::swap(QD[i],QD[j]);
+	}
+
+	~SVC_QMC()
+	{
+		delete[] y;
+		delete cache;
+		delete[] QD;
+	}
+private:
+	float64_t factor;
+	float64_t nr_class;
+	schar *y;
+	Cache *cache;
+	Qfloat *QD;
+};
+
+//
+// Solver for nu-svm classification and regression
+//
+// additional constraint: e^T \alpha = constant
+//
+class Solver_NUMC : public Solver
+{
+public:
+	Solver_NUMC(int32_t n_class, float64_t svm_nu)
+	{
+		nr_class=n_class;
+		nu=svm_nu;
+	}
+
+	void Solve(
+		int32_t p_l, const QMatrix& p_Q, const float64_t *p_p,
+		const schar *p_y, float64_t* p_alpha, float64_t p_Cp, float64_t p_Cn,
+		float64_t p_eps, SolutionInfo* p_si, int32_t shrinking)
+	{
+		this->si = p_si;
+		Solver::Solve(p_l,p_Q,p_p,p_y,p_alpha,p_Cp,p_Cn,p_eps,p_si,shrinking);
+	}
+	float64_t compute_primal(const schar* p_y, float64_t* p_alpha, float64_t* biases, float64_t* normwcw);
+
+private:
+	SolutionInfo *si;
+	int32_t select_working_set(int32_t &i, int32_t &j, float64_t &gap);
+	float64_t calculate_rho();
+	bool be_shrunk(
+		int32_t i, float64_t Gmax1, float64_t Gmax2, float64_t Gmax3,
+		float64_t Gmax4);
+	void do_shrinking();
+
+private:
+	int32_t nr_class;
+	float64_t  nu;
+};
+
+float64_t Solver_NUMC::compute_primal(const schar* p_y, float64_t* p_alpha, float64_t* biases, float64_t* normwcw)
+{
+	clone(y, p_y,l);
+	clone(alpha,p_alpha,l);
+
+	alpha_status = new char[l];
+	for(int32_t i=0;i<l;i++)
+		update_alpha_status(i);
+
+	float64_t* class_count = new float64_t[nr_class];
+	float64_t* outputs = new float64_t[l];
+
+	for (int32_t i=0; i<nr_class; i++)
+	{
+		class_count[i]=0;
+		biases[i+1]=0;
+	}
+
+	for (int32_t i=0; i<active_size; i++)
+	{
+		update_alpha_status(i);
+		if(!is_upper_bound(i) && !is_lower_bound(i))
+			class_count[(int32_t) y[i]]++;
+	}
+
+	//CMath::display_vector(class_count, nr_class, "class_count");
+
+	float64_t mu=((float64_t) nr_class)/(nu*l);
+	//SG_SPRINT("nr_class=%d, l=%d, active_size=%d, nu=%f, mu=%f\n", nr_class, l, active_size, nu, mu);
+
+	float64_t rho=0;
+	float64_t quad=0;
+	float64_t* zero_counts  = new float64_t[nr_class];
+	float64_t normwc_const = 0;
+
+	for (int32_t i=0; i<nr_class; i++)
+	{
+		zero_counts[i]=-INF;
+		normwcw[i]=0;
+	}
+
+	for (int32_t i=0; i<active_size; i++)
+	{
+		float64_t sum_free=0;
+		float64_t sum_atbound=0;
+		float64_t sum_zero_count=0;
+
+		Qfloat* Q_i = Q->get_Q(i,active_size);
+		outputs[i]=0;
+
+		for (int j=0; j<active_size; j++)
+		{
+			quad+= alpha[i]*alpha[j]*Q_i[j];
+			float64_t tmp= alpha[j]*Q_i[j]/mu;
+
+			if(!is_upper_bound(i) && !is_lower_bound(i))
+				sum_free+=tmp;
+			else
+				sum_atbound+=tmp;
+
+			if (class_count[(int32_t) y[i]] == 0 && y[j]==y[i])
+				sum_zero_count+= tmp;
+
+			SVC_QMC* QMC=(SVC_QMC*) Q;
+			float64_t norm_tmp=alpha[i]*alpha[j]*QMC->get_orig_Qij(Q_i[j], i, j);
+			if (y[i]==y[j])
+				normwcw[(int32_t) y[i]]+=norm_tmp;
+
+			normwcw[(int32_t) y[i]]-=2.0/nr_class*norm_tmp;
+			normwc_const+=norm_tmp;
+		}
+
+		if (class_count[(int32_t) y[i]] == 0)
+		{
+			if (zero_counts[(int32_t) y[i]]<sum_zero_count)
+				zero_counts[(int32_t) y[i]]=sum_zero_count;
+		}
+
+		biases[(int32_t) y[i]+1]-=sum_free;
+		if (class_count[(int32_t) y[i]] != 0.0)
+			rho+=sum_free/class_count[(int32_t) y[i]];
+		outputs[i]+=sum_free+sum_atbound;
+	}
+
+	for (int32_t i=0; i<nr_class; i++)
+	{
+		if (class_count[i] == 0.0)
+			rho+=zero_counts[i];
+
+		normwcw[i]+=normwc_const/CMath::sq(nr_class);
+		normwcw[i]=CMath::sqrt(normwcw[i]);
+	}
+
+	CMath::display_vector(normwcw, nr_class, "normwcw");
+
+	rho/=nr_class;
+
+	SG_SPRINT("rho=%f\n", rho);
+
+	float64_t sumb=0;
+	for (int32_t i=0; i<nr_class; i++)
+	{
+		if (class_count[i] != 0.0)
+			biases[i+1]=biases[i+1]/class_count[i]+rho;
+		else
+			biases[i+1]+=rho-zero_counts[i];
+
+		SG_SPRINT("biases=%f\n", biases[i+1]);
+
+		sumb+=biases[i+1];
+	}
+	SG_SPRINT("sumb=%f\n", sumb);
+
+	delete[] zero_counts;
+
+	for (int32_t i=0; i<l; i++)
+		outputs[i]+=biases[(int32_t) y[i]+1];
+
+	biases[0]=rho;
+
+	//CMath::display_vector(outputs, l, "outputs");
+
+
+	float64_t xi=0;
+	for (int32_t i=0; i<active_size; i++)
+	{
+		if (is_lower_bound(i))
+			continue;
+		xi+=rho-outputs[i];
+	}
+
+	//SG_SPRINT("xi=%f\n", xi);
+
+	//SG_SPRINT("quad=%f Cp=%f xi*mu=%f\n", quad, nr_class*rho, xi*mu);
+
+	float64_t primal=0.5*quad- nr_class*rho+xi*mu;
+
+	//SG_SPRINT("primal=%10.10f\n", primal);
+
+	delete[] y;
+	delete[] alpha;
+	delete[] alpha_status;
+
+	return primal;
+}
+
+
+// return 1 if already optimal, return 0 otherwise
+int32_t Solver_NUMC::select_working_set(
+	int32_t &out_i, int32_t &out_j, float64_t &gap)
+{
+	// return i,j such that y_i = y_j and
+	// i: maximizes -y_i * grad(f)_i, i in I_up(\alpha)
+	// j: minimizes the decrease of obj value
+	//    (if quadratic coefficient <= 0, replace it with tau)
+	//    -y_j*grad(f)_j < -y_i*grad(f)_i, j in I_low(\alpha)
+
+	int32_t retval=0;
+	float64_t best_gap=0;
+	int32_t best_out_i=-1;
+	int32_t best_out_j=-1;
+
+	float64_t* Gmaxp = new float64_t[nr_class];
+	float64_t* Gmaxp2 = new float64_t[nr_class];
+	int32_t* Gmaxp_idx = new int32_t[nr_class];
+
+	int32_t* Gmin_idx = new int32_t[nr_class];
+	float64_t* obj_diff_min = new float64_t[nr_class];
+
+	for (int32_t i=0; i<nr_class; i++)
+	{
+		Gmaxp[i]=-INF;
+		Gmaxp2[i]=-INF;
+		Gmaxp_idx[i]=-1;
+		Gmin_idx[i]=-1;
+		obj_diff_min[i]=INF;
+	}
+
+	for(int32_t t=0;t<active_size;t++)
+	{
+		int32_t cidx=y[t];
+		if(!is_upper_bound(t))
+		{
+			if(-G[t] >= Gmaxp[cidx])
+			{
+				Gmaxp[cidx] = -G[t];
+				Gmaxp_idx[cidx] = t;
+			}
+		}
+	}
+
+	for(int32_t j=0;j<active_size;j++)
+	{
+		int32_t cidx=y[j];
+		int32_t ip = Gmaxp_idx[cidx];
+		const Qfloat *Q_ip = NULL;
+		if(ip != -1) // NULL Q_ip not accessed: Gmaxp=-INF if ip=-1
+			Q_ip = Q->get_Q(ip,active_size);
+
+		if (!is_lower_bound(j))
+		{
+			float64_t grad_diff=Gmaxp[cidx]+G[j];
+			if (G[j] >= Gmaxp2[cidx])
+				Gmaxp2[cidx] = G[j];
+			if (grad_diff > 0)
+			{
+				float64_t obj_diff;
+				float64_t quad_coef = Q_ip[ip]+QD[j]-2*Q_ip[j];
+				if (quad_coef > 0)
+					obj_diff = -(grad_diff*grad_diff)/quad_coef;
+				else
+					obj_diff = -(grad_diff*grad_diff)/TAU;
+
+				if (obj_diff <= obj_diff_min[cidx])
+				{
+					Gmin_idx[cidx]=j;
+					obj_diff_min[cidx] = obj_diff;
+				}
+			}
+		}
+
+		gap=Gmaxp[cidx]+Gmaxp2[cidx];
+		if (gap>=best_gap && Gmin_idx[cidx]>=0 &&
+				Gmaxp_idx[cidx]>=0 && Gmin_idx[cidx]<active_size)
+		{
+			out_i = Gmaxp_idx[cidx];
+			out_j = Gmin_idx[cidx];
+
+			best_gap=gap;
+			best_out_i=out_i;
+			best_out_j=out_j;
+		}
+	}
+
+	gap=best_gap;
+	out_i=best_out_i;
+	out_j=best_out_j;
+
+	SG_SDEBUG("i=%d j=%d best_gap=%f y_i=%f y_j=%f\n", out_i, out_j, gap, y[out_i], y[out_j]);
+
+
+	if(gap < eps)
+		retval=1;
+
+	delete[] Gmaxp;
+	delete[] Gmaxp2;
+	delete[] Gmaxp_idx;
+	delete[] Gmin_idx;
+	delete[] obj_diff_min;
+
+	return retval;
+}
+
+bool Solver_NUMC::be_shrunk(
+	int32_t i, float64_t Gmax1, float64_t Gmax2, float64_t Gmax3,
+	float64_t Gmax4)
+{
+	return false;
+}
+
+void Solver_NUMC::do_shrinking()
+{
+}
+
+float64_t Solver_NUMC::calculate_rho()
+{
+	return 0;
+}
+
+
 //
 // Q matrices for various formulations
 //
 class SVC_Q: public LibSVMKernel
-{ 
+{
 public:
 	SVC_Q(const svm_problem& prob, const svm_parameter& param, const schar *y_)
 	:LibSVMKernel(prob.l, prob.x, param)
@@ -1147,7 +1577,7 @@ public:
 		for(int32_t i=0;i<prob.l;i++)
 			QD[i]= (Qfloat)kernel_function(i,i);
 	}
-	
+
 	Qfloat *get_Q(int32_t i, int32_t len) const
 	{
 		Qfloat *data;
@@ -1169,8 +1599,8 @@ public:
 	{
 		cache->swap_index(i,j);
 		LibSVMKernel::swap_index(i,j);
-		swap(y[i],y[j]);
-		swap(QD[i],QD[j]);
+		CMath::swap(y[i],y[j]);
+		CMath::swap(QD[i],QD[j]);
 	}
 
 	~SVC_Q()
@@ -1185,6 +1615,7 @@ private:
 	Qfloat *QD;
 };
 
+
 class ONE_CLASS_Q: public LibSVMKernel
 {
 public:
@@ -1196,7 +1627,7 @@ public:
 		for(int32_t i=0;i<prob.l;i++)
 			QD[i]= (Qfloat)kernel_function(i,i);
 	}
-	
+
 	Qfloat *get_Q(int32_t i, int32_t len) const
 	{
 		Qfloat *data;
@@ -1218,7 +1649,7 @@ public:
 	{
 		cache->swap_index(i,j);
 		LibSVMKernel::swap_index(i,j);
-		swap(QD[i],QD[j]);
+		CMath::swap(QD[i],QD[j]);
 	}
 
 	~ONE_CLASS_Q()
@@ -1232,7 +1663,7 @@ private:
 };
 
 class SVR_Q: public LibSVMKernel
-{ 
+{
 public:
 	SVR_Q(const svm_problem& prob, const svm_parameter& param)
 	:LibSVMKernel(prob.l, prob.x, param)
@@ -1258,11 +1689,11 @@ public:
 
 	void swap_index(int32_t i, int32_t j) const
 	{
-		swap(sign[i],sign[j]);
-		swap(index[i],index[j]);
-		swap(QD[i],QD[j]);
+		CMath::swap(sign[i],sign[j]);
+		CMath::swap(index[i],index[j]);
+		CMath::swap(QD[i],QD[j]);
 	}
-	
+
 	Qfloat *get_Q(int32_t i, int32_t len) const
 	{
 		Qfloat *data;
@@ -1315,7 +1746,6 @@ static void solve_c_svc(
 	float64_t *alpha, Solver::SolutionInfo* si, float64_t Cp, float64_t Cn)
 {
 	int32_t l = prob->l;
-	float64_t *minus_ones = new float64_t[l];
 	schar *y = new schar[l];
 
 	int32_t i;
@@ -1323,12 +1753,11 @@ static void solve_c_svc(
 	for(i=0;i<l;i++)
 	{
 		alpha[i] = 0;
-		minus_ones[i] = -1;
 		if(prob->y[i] > 0) y[i] = +1; else y[i]=-1;
 	}
 
 	Solver s;
-	s.Solve(l, SVC_Q(*prob,*param,y), minus_ones, y,
+	s.Solve(l, SVC_Q(*prob,*param,y), prob->pv, y,
 		alpha, Cp, Cn, param->eps, si, param->shrinking);
 
 	float64_t sum_alpha=0;
@@ -1338,6 +1767,42 @@ static void solve_c_svc(
 	if (Cp==Cn)
 		SG_SINFO("nu = %f\n", sum_alpha/(param->C*prob->l));
 
+	for(i=0;i<l;i++)
+		alpha[i] *= y[i];
+
+	delete[] y;
+}
+
+
+//two weighted datasets
+void solve_c_svc_weighted(
+	const svm_problem *prob, const svm_parameter* param,
+	float64_t *alpha, Solver::SolutionInfo* si, float64_t Cp, float64_t Cn)
+{
+	int l = prob->l;
+	float64_t *minus_ones = new float64_t[l];
+	schar *y = new schar[l];
+
+	int i;
+
+	for(i=0;i<l;i++)
+	{
+		alpha[i] = 0;
+		minus_ones[i] = -1;
+		if(prob->y[i] > 0) y[i] = +1; else y[i]=-1;
+	}
+
+	WeightedSolver s = WeightedSolver(prob->C);
+	s.Solve(l, SVC_Q(*prob,*param,y), minus_ones, y,
+		alpha, Cp, Cn, param->eps, si, param->shrinking);
+
+	float64_t sum_alpha=0;
+	for(i=0;i<l;i++)
+		sum_alpha += alpha[i];
+
+	//if (Cp==Cn)
+	//	SG_SINFO("nu = %f\n", sum_alpha/(prob->C*prob->l));
+
 	for(i=0;i<l;i++)
 		alpha[i] *= y[i];
 
@@ -1367,12 +1832,12 @@ static void solve_nu_svc(
 	for(i=0;i<l;i++)
 		if(y[i] == +1)
 		{
-			alpha[i] = min(1.0,sum_pos);
+			alpha[i] = CMath::min(1.0,sum_pos);
 			sum_pos -= alpha[i];
 		}
 		else
 		{
-			alpha[i] = min(1.0,sum_neg);
+			alpha[i] = CMath::min(1.0,sum_neg);
 			sum_neg -= alpha[i];
 		}
 
@@ -1400,6 +1865,103 @@ static void solve_nu_svc(
 	delete[] zeros;
 }
 
+static void solve_nu_multiclass_svc(const svm_problem *prob,
+		const svm_parameter *param, Solver::SolutionInfo* si, svm_model* model)
+{
+	int32_t l = prob->l;
+	float64_t nu = param->nu;
+
+	float64_t *alpha = Malloc(float64_t, prob->l);
+	schar *y = new schar[l];
+
+	for(int32_t i=0;i<l;i++)
+	{
+		alpha[i] = 0;
+		y[i]=prob->y[i];
+	}
+
+	int32_t nr_class=param->nr_class;
+	float64_t* sum_class = new float64_t[nr_class];
+
+	for (int32_t j=0; j<nr_class; j++)
+		sum_class[j] = nu*l/nr_class;
+
+	for(int32_t i=0;i<l;i++)
+	{
+		alpha[i] = CMath::min(1.0,sum_class[int32_t(y[i])]);
+		sum_class[int32_t(y[i])] -= alpha[i];
+	}
+	delete[] sum_class;
+
+
+	float64_t *zeros = new float64_t[l];
+
+	for (int32_t i=0;i<l;i++)
+		zeros[i] = 0;
+
+	Solver_NUMC s(nr_class, nu);
+	SVC_QMC Q(*prob,*param,y, nr_class, ((float64_t) nr_class)/CMath::sq(nu*l));
+
+	s.Solve(l, Q, zeros, y,
+		alpha, 1.0, 1.0, param->eps, si,  param->shrinking);
+
+
+	int32_t* class_sv_count=new int32_t[nr_class];
+
+	for (int32_t i=0; i<nr_class; i++)
+		class_sv_count[i]=0;
+
+	for (int32_t i=0; i<l; i++)
+	{
+		if (CMath::abs(alpha[i]) > 0)
+			class_sv_count[(int32_t) y[i]]++;
+	}
+
+	model->l=l;
+	// rho[0]= rho in mcsvm paper, rho[1]...rho[nr_class] is bias in mcsvm paper
+	model->rho = Malloc(float64_t, nr_class+1);
+	model->nr_class = nr_class;
+	model->label = NULL;
+	model->SV = Malloc(svm_node*,nr_class);
+	model->nSV = Malloc(int32_t, nr_class);
+	model->sv_coef = Malloc(float64_t *,nr_class);
+	model->normwcw = Malloc(float64_t,nr_class);
+
+	for (int32_t i=0; i<nr_class+1; i++)
+		model->rho[i]=0;
+
+	model->objective = si->obj;
+
+	if (param->use_bias)
+	{
+		SG_SDEBUG("Computing biases and primal objective\n");
+		float64_t primal = s.compute_primal(y, alpha, model->rho, model->normwcw);
+		SG_SINFO("Primal = %10.10f\n", primal);
+	}
+
+	for (int32_t i=0; i<nr_class; i++)
+	{
+		model->nSV[i]=class_sv_count[i];
+		model->SV[i] = Malloc(svm_node,class_sv_count[i]);
+		model->sv_coef[i] = Malloc(float64_t,class_sv_count[i]);
+		class_sv_count[i]=0;
+	}
+
+	for (int32_t i=0;i<prob->l;i++)
+	{
+		if(fabs(alpha[i]) > 0)
+		{
+			model->SV[(int32_t) y[i]][class_sv_count[(int32_t) y[i]]].index = prob->x[i]->index;
+			model->sv_coef[(int32_t) y[i]][class_sv_count[(int32_t) y[i]]] = alpha[i];
+			class_sv_count[(int32_t) y[i]]++;
+		}
+	}
+
+	delete[] y;
+	delete[] zeros;
+	free(alpha);
+}
+
 static void solve_one_class(
 	const svm_problem *prob, const svm_parameter *param,
 	float64_t *alpha, Solver::SolutionInfo* si)
@@ -1484,7 +2046,7 @@ static void solve_nu_svr(
 	float64_t sum = C * param->nu * l / 2;
 	for(i=0;i<l;i++)
 	{
-		alpha2[i] = alpha2[i+l] = min(sum,C);
+		alpha2[i] = alpha2[i+l] = CMath::min(sum,C);
 		sum -= alpha2[i];
 
 		linear_term[i] = - prob->y[i];
@@ -1514,7 +2076,7 @@ static void solve_nu_svr(
 struct decision_function
 {
 	float64_t *alpha;
-	float64_t rho;	
+	float64_t rho;
 	float64_t objective;
 };
 
@@ -1678,10 +2240,16 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 				model->SV[j] = prob->x[i];
 				model->sv_coef[0][j] = f.alpha[i];
 				++j;
-			}		
+			}
 
 		free(f.alpha);
 	}
+	else if(param->svm_type == NU_MULTICLASS_SVC)
+	{
+		Solver::SolutionInfo si;
+		solve_nu_multiclass_svc(prob,param,&si,model);
+		SG_SINFO("obj = %.16f, rho = %.16f\n",si.obj,si.rho);
+	}
 	else
 	{
 		// classification
@@ -1695,13 +2263,30 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 		// group training data of the same class
 		svm_group_classes(prob,&nr_class,&label,&start,&count,perm);
 		svm_node **x = Malloc(svm_node *,l);
+		float64_t *C = Malloc(float64_t,l);
+		float64_t *pv = Malloc(float64_t,l);
+
+
 		int32_t i;
-		for(i=0;i<l;i++)
+		for(i=0;i<l;i++) {
 			x[i] = prob->x[perm[i]];
+            C[i] = prob->C[perm[i]];
+
+            if (prob->pv)
+            {
+            	pv[i] = prob->pv[perm[i]];
+            }
+            else
+            {
+				//no custom linear term is set
+            	pv[i] = -1.0;
+            }
+
+		}
+
 
 		// calculate weighted C
-
-		float64_t *weighted_C = Malloc(float64_t,  nr_class);
+		float64_t *weighted_C = Malloc(float64_t, nr_class);
 		for(i=0;i<nr_class;i++)
 			weighted_C[i] = param->C;
 		for(i=0;i<param->nr_weight;i++)
@@ -1717,7 +2302,7 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 		}
 
 		// train k*(k-1)/2 models
-		
+
 		bool *nonzero = Malloc(bool,l);
 		for(i=0;i<l;i++)
 			nonzero[i] = false;
@@ -1732,21 +2317,30 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 				int32_t ci = count[i], cj = count[j];
 				sub_prob.l = ci+cj;
 				sub_prob.x = Malloc(svm_node *,sub_prob.l);
-				sub_prob.y = Malloc(float64_t, sub_prob.l+1); //dirty hack to surpress valgrind err
+				sub_prob.y = Malloc(float64_t,sub_prob.l+1); //dirty hack to surpress valgrind err
+				sub_prob.C = Malloc(float64_t,sub_prob.l+1);
+				sub_prob.pv = Malloc(float64_t,sub_prob.l+1);
 
 				int32_t k;
 				for(k=0;k<ci;k++)
 				{
 					sub_prob.x[k] = x[si+k];
 					sub_prob.y[k] = +1;
+                    sub_prob.C[k] = C[si+k];
+                    sub_prob.pv[k] = pv[si+k];
+
 				}
 				for(k=0;k<cj;k++)
 				{
 					sub_prob.x[ci+k] = x[sj+k];
 					sub_prob.y[ci+k] = -1;
+                    sub_prob.C[ci+k] = C[sj+k];
+                    sub_prob.pv[ci+k] = pv[sj+k];
 				}
 				sub_prob.y[sub_prob.l]=-1; //dirty hack to surpress valgrind err
-				
+				sub_prob.C[sub_prob.l]=-1;
+				sub_prob.pv[sub_prob.l]=-1;
+
 				f[p] = svm_train_one(&sub_prob,param,weighted_C[i],weighted_C[j]);
 				for(k=0;k<ci;k++)
 					if(!nonzero[si+k] && fabs(f[p].alpha[k]) > 0)
@@ -1756,6 +2350,8 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 						nonzero[sj+k] = true;
 				free(sub_prob.x);
 				free(sub_prob.y);
+				free(sub_prob.C);
+				free(sub_prob.pv);
 				++p;
 			}
 
@@ -1763,11 +2359,11 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 
 		model->objective = f[0].objective;
 		model->nr_class = nr_class;
-		
+
 		model->label = Malloc(int32_t, nr_class);
 		for(i=0;i<nr_class;i++)
 			model->label[i] = label[i];
-		
+
 		model->rho = Malloc(float64_t, nr_class*(nr_class-1)/2);
 		for(i=0;i<nr_class*(nr_class-1)/2;i++)
 			model->rho[i] = f[i].rho;
@@ -1780,14 +2376,14 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 			int32_t nSV = 0;
 			for(int32_t j=0;j<count[i];j++)
 				if(nonzero[start[i]+j])
-				{	
+				{
 					++nSV;
 					++total_sv;
 				}
 			model->nSV[i] = nSV;
 			nz_count[i] = nSV;
 		}
-		
+
 		SG_SINFO("Total nSV = %d\n",total_sv);
 
 		model->l = total_sv;
@@ -1829,12 +2425,14 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 						model->sv_coef[i][q++] = f[p].alpha[ci+k];
 				++p;
 			}
-		
+
 		free(label);
 		free(count);
 		free(perm);
 		free(start);
 		free(x);
+		free(C);
+		free(pv);
 		free(weighted_C);
 		free(nonzero);
 		for(i=0;i<nr_class*(nr_class-1)/2;i++)
@@ -1846,16 +2444,6 @@ svm_model *svm_train(const svm_problem *prob, const svm_parameter *param)
 	return model;
 }
 
-const char *svm_type_table[] =
-{
-	"c_svc","nu_svc","one_class","epsilon_svr","nu_svr",NULL
-};
-
-const char *kernel_type_table[]=
-{
-	"linear","polynomial","rbf","sigmoid","precomputed",NULL
-};
-
 void svm_destroy_model(svm_model* model)
 {
 	if(model->free_sv && model->l > 0)
@@ -1886,7 +2474,8 @@ const char *svm_check_parameter(
 	   svm_type != NU_SVC &&
 	   svm_type != ONE_CLASS &&
 	   svm_type != EPSILON_SVR &&
-	   svm_type != NU_SVR)
+	   svm_type != NU_SVR &&
+	   svm_type != NU_MULTICLASS_SVC)
 		return "unknown svm type";
 
 	// kernel_type, degree
@@ -1932,7 +2521,7 @@ const char *svm_check_parameter(
 
 
 	// check whether nu-svc is feasible
-	
+
 	if(svm_type == NU_SVC)
 	{
 		int32_t l = prob->l;
@@ -1967,14 +2556,14 @@ const char *svm_check_parameter(
 				++nr_class;
 			}
 		}
-	
+
 		for(i=0;i<nr_class;i++)
 		{
 			int32_t n1 = count[i];
 			for(int32_t j=i+1;j<nr_class;j++)
 			{
 				int32_t n2 = count[j];
-				if(param->nu*(n1+n2)/2 > min(n1,n2))
+				if(param->nu*(n1+n2)/2 > CMath::min(n1,n2))
 				{
 					free(label);
 					free(count);
@@ -1988,4 +2577,5 @@ const char *svm_check_parameter(
 
 	return NULL;
 }
+}
 #endif // DOXYGEN_SHOULD_SKIP_THIS
