@@ -7,6 +7,8 @@
 #include "base/SGObject.h"
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
+namespace shogun
+{
 /** class function */
 class function
 {
@@ -81,6 +83,6 @@ private:
 	int32_t max_iter;
 	function *fun_obj;
 };
-
+}
 #endif
 #endif //HAVE_LAPACK
