@@ -58,12 +58,12 @@
 #include "classifier/svm/qpbsvmlib.h"
 #include "classifier/svm/pr_loqo.h"
 
+using namespace shogun;
+
 #define HISTORY_BUF 1000000
 
 #define INDEX(ROW,COL,DIM) ((COL*DIM)+ROW)
 
-float64_t sparsity=0;
-
 CQPBSVMLib::CQPBSVMLib(
 	float64_t* H, int32_t n, float64_t* f, int32_t m, float64_t UB)
 : CSGObject()
@@ -595,7 +595,6 @@ int32_t CQPBSVMLib::qpbsvm_gauss_seidel(float64_t *x,
 			atbound++;
 	}
 	SG_PRINT("atbound:%d of %d (%2.2f%%)\n", atbound, m_dim, ((float64_t) 100.0*atbound)/m_dim);
-	sparsity+=((float64_t) 100.0*atbound)/m_dim;
 	*ptr_t=0;
 	*ptr_History=NULL;
 	return 0;
@@ -626,7 +625,6 @@ int32_t CQPBSVMLib::qpbsvm_gradient_descent(float64_t *x,
 			atbound++;
 	}
 	SG_PRINT("atbound:%d of %d (%2.2f%%)\n", atbound, m_dim, ((float64_t) 100.0*atbound)/m_dim);
-	sparsity+=((float64_t) 100.0*atbound)/m_dim;
 	*ptr_t=0;
 	*ptr_History=NULL;
 	return 0;
