@@ -28,8 +28,8 @@
 #include "features/SparseFeatures.h"
 #include "classifier/svm/ssl.h"
 
-#define VERBOSE 1
-
+namespace shogun
+{
 void ssl_train(struct data *Data, 
 		struct options *Options, 
 		struct vector_double *Weights,
@@ -1133,3 +1133,4 @@ void GetLabeledData(struct data *D, const struct data *Data)
 	D->u=0;
 	delete [] J;*/
 }
+}
