@@ -23,6 +23,8 @@
 
 #include <string.h>
 
+namespace shogun
+{
 template <class ST> class CStringFeatures;
 template <class ST> class CSimpleFeatures;
 template <class ST> class CSimplePreProc;
@@ -620,6 +622,67 @@ template <class ST> class CSimpleFeatures: public CDotFeatures
 		/** @return object name */
 		inline virtual const char* get_name() const { return "SimpleFeatures"; }
 
+
+#ifdef HAVE_BOOST_SERIALIZATION
+    private:
+
+        /// serialization needs to split up in save/load because 
+        ///  the serialization of pointers to natives (int* & friends) 
+        ///  requires a workaround 
+        friend class ::boost::serialization::access;
+        template<class Archive>
+            void save(Archive & ar, const unsigned int archive_version) const
+            {
+
+                SG_DEBUG("archiving SimpleFeatures\n");
+
+                ar & ::boost::serialization::base_object<CDotFeatures>(*this);
+
+                //TODO
+                //ar & feature_cache;
+
+                ar & num_vectors;
+                ar & num_features;
+
+                for (int i=0; i < num_vectors*num_features; ++i) 
+                {
+                    ar & feature_matrix[i];
+                }
+
+                SG_DEBUG("done SimpleFeatures\n");
+
+
+            }
+
+        template<class Archive>
+            void load(Archive & ar, const unsigned int archive_version)
+            {
+
+                SG_DEBUG("archiving SimpleFeatures\n");
+
+                ar & ::boost::serialization::base_object<CDotFeatures>(*this);
+
+                //TODO
+                //ar & feature_cache;
+
+                ar & num_vectors;
+                ar & num_features;
+
+                feature_matrix = new ST[num_vectors*num_features];
+                for (int i=0; i< num_vectors*num_features; ++i){
+                    ar & feature_matrix[i];
+                }
+
+
+                SG_DEBUG("done SimpleFeatures\n");
+
+            }
+
+        GLOBAL_BOOST_SERIALIZATION_SPLIT_MEMBER();
+
+#endif //HAVE_BOOST_SERIALIZATION
+
+
 	protected:
 		/** compute feature vector for sample num
 		 * if target is set the vector is written to target
@@ -833,6 +896,14 @@ template<> inline const char* CSimpleFeatures<floatmax_t>::get_name() const
 	return "LongRealFeatures";
 }
 
+/** align strings and compute emperical kernel map based on alignment scores
+ *
+ * non functional code - needs updating
+ *
+ * @param cf strings to be aligned to reference
+ * @param Ref reference strings to be aligned to
+ * @param gapCost costs for a gap
+ */
 template<> inline bool CSimpleFeatures<float64_t>::Align_char_features(
 		CStringFeatures<char>* cf, CStringFeatures<char>* Ref, float64_t gapCost)
 {
@@ -1092,4 +1163,5 @@ template<> inline float64_t CSimpleFeatures<floatmax_t>:: dense_dot(int32_t vec_
 	return result;
 }
 #endif // DOXYGEN_SHOULD_SKIP_THIS
+}
 #endif // _SIMPLEFEATURES__H__
