@@ -27,6 +27,13 @@
 #include "features/SimpleFeatures.h"
 #include "preproc/SparsePreProc.h"
 
+namespace shogun
+{
+
+class CLabels;
+class CFeatures;
+class CDotFeatures;
+template <class ST> class CSimpleFeatures;
 template <class ST> class CSparsePreProc;
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
@@ -82,12 +89,25 @@ template <class ST> class CSparseFeatures : public CDotFeatures
 		 * @param src dense feature matrix
 		 * @param num_feat number of features
 		 * @param num_vec number of vectors
+		 * @param copy true to copy feature matrix
 		 */
-		CSparseFeatures(TSparse<ST>* src, int32_t num_feat, int32_t num_vec)
+		CSparseFeatures(TSparse<ST>* src, int32_t num_feat, int32_t num_vec, bool copy=false)
 		: CDotFeatures(0), num_vectors(0), num_features(0),
 			sparse_feature_matrix(NULL), feature_cache(NULL)
 		{
-			set_sparse_feature_matrix(src, num_feat, num_vec);
+			if (!copy)
+				set_sparse_feature_matrix(src, num_feat, num_vec);
+			else
+			{
+				sparse_feature_matrix = new TSparse<ST>[num_vec];
+				memcpy(sparse_feature_matrix, src, sizeof(TSparse<ST>)*num_vec);
+				for (int32_t i=0; i< num_vec; i++)
+				{
+					sparse_feature_matrix[i].features = new TSparseEntry<ST>[sparse_feature_matrix[i].num_feat_entries];
+					memcpy(sparse_feature_matrix[i].features, src[i].features, sizeof(TSparseEntry<ST>)*sparse_feature_matrix[i].num_feat_entries);
+
+				}
+			}
 		}
 
 		/** convenience constructor that creates sparse features from
@@ -169,6 +189,37 @@ template <class ST> class CSparseFeatures : public CDotFeatures
 			return new CSparseFeatures<ST>(*this);
 		}
 
+		/** get a single feature
+		 *
+		 * @param num number of feature vector to retrieve
+		 * @param index index of feature in this vector
+		 *
+		 * @return sum of features that match dimension index and 0 if none is found
+		 */
+		ST get_feature(int32_t num, int32_t index)
+		{
+			ASSERT(index>=0 && index<num_features) ;
+			ASSERT(num>=0 && num<num_vectors) ;
+
+			bool vfree;
+			int32_t num_feat;
+			int32_t i;
+			TSparseEntry<ST>* sv=get_sparse_feature_vector(num, num_feat, vfree);
+			ST ret = 0 ;
+			
+			if (sv)
+			{
+				for (i=0; i<num_feat; i++)
+					if (sv[i].feat_index==index)
+						ret += sv[i].entry ;
+			}
+			
+			free_sparse_feature_vector(sv, num, vfree);
+			
+			return ret ;
+		}
+		
+
 		/** converts a sparse feature vector into a dense one
 		  * preprocessed compute_feature_vector
 		  * caller cleans up
@@ -966,9 +1017,11 @@ template <class ST> class CSparseFeatures : public CDotFeatures
 		/** load features from file
 		 *
 		 * @param fname filename to load from
+		 * @param do_sort_features if true features will be sorted to ensure they
+		 * 		 are in ascending order
 		 * @return label object with corresponding labels
 		 */
-		CLabels* load_svmlight_file(char* fname)
+		CLabels* load_svmlight_file(char* fname, bool do_sort_features=true)
 		{
 			CLabels* lab=NULL;
 
@@ -1127,9 +1180,56 @@ template <class ST> class CSparseFeatures : public CDotFeatures
 
 			delete[] dummy;
 
+			if (do_sort_features)
+				sort_features();
+
 			return lab;
 		}
 
+		/** ensure that features occur in ascending order, only call when no
+		 * preprocessors are attached */
+		void sort_features()
+		{
+			ASSERT(get_num_preproc()==0);
+
+			if (!sparse_feature_matrix)
+				SG_ERROR("Requires sparse feature matrix to be available in-memory\n");
+
+			for (int32_t i=0; i<num_vectors; i++)
+			{
+				int32_t len=sparse_feature_matrix[i].num_feat_entries;
+
+				if (!len)
+					continue;
+
+				TSparseEntry<ST>* sf_orig=sparse_feature_matrix[i].features;
+				int32_t* feat_idx=new int32_t[len];
+				int32_t* orig_idx=new int32_t[len];
+
+				for (int j=0; j<len; j++)
+				{
+					feat_idx[j]=sf_orig[j].feat_index;
+					orig_idx[j]=j;
+				}
+
+				CMath::qsort_index(feat_idx, orig_idx, len);
+
+				TSparseEntry<ST>* sf_new= new TSparseEntry<ST>[len];
+				for (int j=0; j<len; j++)
+					sf_new[j]=sf_orig[orig_idx[j]];
+
+				sparse_feature_matrix[i].features=sf_new;
+
+				// sanity check
+				for (int j=0; j<len-1; j++)
+					ASSERT(sf_new[j].feat_index<sf_new[j+1].feat_index);
+
+				delete[] orig_idx;
+				delete[] feat_idx;
+				delete[] sf_orig;
+			}
+		}
+
 		/** write features to file using svm light format
 		 *
 		 * @param fname filename to write to
@@ -1371,4 +1471,5 @@ template<> inline EFeatureType CSparseFeatures<floatmax_t>::get_feature_type()
 {
 	return F_LONGREAL;
 }
+}
 #endif /* _SPARSEFEATURES__H__ */
