@@ -12,17 +12,18 @@
 #ifndef _CSTRINGFEATURES__H__
 #define _CSTRINGFEATURES__H__
 
+#include "lib/common.h"
+#include "lib/io.h"
+#include "lib/Cache.h"
 #include "preproc/PreProc.h"
 #include "preproc/StringPreProc.h"
 #include "features/Features.h"
-#include "features/SimpleFeatures.h"
 #include "features/Alphabet.h"
-#include "lib/common.h"
-#include "lib/io.h"
 #include "lib/DynamicArray.h"
 #include "lib/File.h"
 #include "lib/MemoryMappedFile.h"
 #include "lib/Mathematics.h"
+#include "lib/Compressor.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -31,14 +32,71 @@
 #include <stdlib.h>
 #include <unistd.h>
 
+namespace shogun
+{
+class CCompressor;
+enum E_COMPRESSION_TYPE;
+class CAlphabet;
+enum EAlphabet;
+template <class T> class CDynamicArray;
 class CFile;
-
+template <class T> class CMemoryMappedFile;
+class CMath;
 template <class ST> class CStringPreProc;
+template <class T> class T_STRING;
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 /** template class T_STRING */
-template <class T> struct T_STRING
+template <class T> class T_STRING
 {
+
+
+#ifdef HAVE_BOOST_SERIALIZATION
+
+  private:
+
+
+  friend class ::boost::serialization::access;
+  template<class Archive>
+  void save(Archive & ar, const unsigned int archive_version) const
+  {
+
+    //SG_DEBUG("archiving T_STRING\n");
+
+  	ar & length;
+
+    for (int i=0; i < length; ++i) {
+      ar & string[i];
+    }
+
+    //SG_DEBUG("done archiving T_STRING\n");
+
+  }
+
+  template<class Archive>
+  void load(Archive & ar, const unsigned int archive_version)
+  {
+
+	//SG_DEBUG("archiving T_STRING\n");
+
+  	ar & length;
+
+    string = new T[length];
+
+    for (int i=0; i < length; ++i) {
+      ar & string[i];
+    }
+
+    //SG_DEBUG("done archiving T_STRING\n");
+
+  }
+
+  GLOBAL_BOOST_SERIALIZATION_SPLIT_MEMBER();
+
+
+#endif //HAVE_BOOST_SERIALIZATION
+
+    public:
 	/** string */
 	T* string;
 	/** length of string */
@@ -46,31 +104,38 @@ template <class T> struct T_STRING
 };
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 
-template <class T> char* get_zero_terminated_string_copy(T_STRING<T> str)
-{
-	int32_t l=str.length;
-	char* s=new char[l+1];
-	memcpy(s, str.string, sizeof(char)*l);
-	s[l]='\0';
-	return s;
-}
 
 /** @brief Template class StringFeatures implements a list of strings.
  *
- * As this class is template the underlying storage type is quite arbitrary and
+ * As this class is a template the underlying storage type is quite arbitrary and
  * not limited to character strings, but could also be sequences of floating
  * point numbers etc. Strings differ from matrices (cf. CSimpleFeatures) in a
  * way that the dimensionality of the feature vectors (i.e. the strings) is not
  * fixed; it may vary between strings.
- * 
+ *
  * Most string kernels require StringFeatures but a number of them actually
  * requires strings to have same length.
  *
- * Note: StringFeatures do not support PreProcs
+ * When preprocessors are attached to string features they may shorten the
+ * string, but are not allowed to return strings longer than max_string_length,
+ * as some algorithms depend on this.
+ *
+ * Also note that string features cannot currently be computed on-the-fly.
+ *
  */
 template <class ST> class CStringFeatures : public CFeatures
 {
 	public:
+		/** default constructor
+		 *
+		 */
+        CStringFeatures() : CFeatures(0), alphabet(NULL), num_vectors(0),
+		features(NULL), single_string(NULL),length_of_single_string(0),
+        max_string_length(0), order(0), symbol_mask_table(NULL),
+		preprocess_on_get(false), feature_cache(NULL)
+        {
+        }
+
 		/** constructor
 		 *
 		 * @param alpha alphabet (type) to use for string features
@@ -78,7 +143,8 @@ template <class ST> class CStringFeatures : public CFeatures
 		CStringFeatures(EAlphabet alpha)
 		: CFeatures(0), num_vectors(0), features(NULL),
 			single_string(NULL),length_of_single_string(0),
-			max_string_length(0), order(0), symbol_mask_table(NULL)
+			max_string_length(0), order(0), symbol_mask_table(NULL),
+			preprocess_on_get(false), feature_cache(NULL)
 		{
 			alphabet=new CAlphabet(alpha);
 			SG_REF(alphabet);
@@ -97,7 +163,8 @@ template <class ST> class CStringFeatures : public CFeatures
 				int32_t p_max_string_length, EAlphabet alpha)
 		: CFeatures(0), num_vectors(0), features(NULL),
 			single_string(NULL),length_of_single_string(0),
-			max_string_length(0), order(0), symbol_mask_table(NULL)
+			max_string_length(0), order(0), symbol_mask_table(NULL),
+			preprocess_on_get(false), feature_cache(NULL)
 		{
 			alphabet=new CAlphabet(alpha);
 			SG_REF(alphabet);
@@ -113,7 +180,8 @@ template <class ST> class CStringFeatures : public CFeatures
 		CStringFeatures(CAlphabet* alpha)
 		: CFeatures(0), num_vectors(0), features(NULL),
 			single_string(NULL),length_of_single_string(0),
-			max_string_length(0), order(0), symbol_mask_table(NULL)
+			max_string_length(0), order(0), symbol_mask_table(NULL),
+			preprocess_on_get(false), feature_cache(NULL)
 		{
 			ASSERT(alpha);
 			SG_REF(alpha);
@@ -130,7 +198,8 @@ template <class ST> class CStringFeatures : public CFeatures
 			max_string_length(orig.max_string_length),
 			num_symbols(orig.num_symbols),
 			original_num_symbols(orig.original_num_symbols),
-			order(orig.order)
+			order(orig.order), preprocess_on_get(false),
+			feature_cache(NULL)
 		{
 			ASSERT(orig.single_string == NULL); //not implemented
 
@@ -146,7 +215,7 @@ template <class ST> class CStringFeatures : public CFeatures
 					features[i].string=new ST[orig.features[i].length];
 					ASSERT(features[i].string);
 					features[i].length=orig.features[i].length;
-					memcpy(features[i].string, orig.features[i].string, sizeof(ST)*orig.features[i].length); 
+					memcpy(features[i].string, orig.features[i].string, sizeof(ST)*orig.features[i].length);
 				}
 			}
 
@@ -164,10 +233,9 @@ template <class ST> class CStringFeatures : public CFeatures
 		 * @param alpha alphabet (type) to use for string features
 		 */
 		CStringFeatures(char* fname, EAlphabet alpha=DNA)
-		: CFeatures(fname), num_vectors(0),
-			features(NULL), single_string(NULL),
-			length_of_single_string(0), max_string_length(0),
-			order(0), symbol_mask_table(NULL)
+		: CFeatures(fname), num_vectors(0), features(NULL), single_string(NULL),
+			length_of_single_string(0), max_string_length(0), order(0),
+			symbol_mask_table(NULL), preprocess_on_get(false), feature_cache(NULL)
 		{
 			alphabet=new CAlphabet(alpha);
 			SG_REF(alphabet);
@@ -184,7 +252,7 @@ template <class ST> class CStringFeatures : public CFeatures
 		}
 
 		/** cleanup string features */
-		void cleanup()
+		virtual void cleanup()
 		{
 			if (single_string)
 			{
@@ -194,10 +262,7 @@ template <class ST> class CStringFeatures : public CFeatures
 			else
 			{
 				for (int32_t i=0; i<num_vectors; i++)
-				{
-					delete[] features[i].string;
-					features[i].length=0;
-				}
+					cleanup_feature_vector(i);
 			}
 
 			num_vectors=0;
@@ -208,7 +273,7 @@ template <class ST> class CStringFeatures : public CFeatures
 
 			/* start with a fresh alphabet, but instead of emptying the histogram
 			 * create a new object (to leave the alphabet object alone if it is used
-			 * by others) 
+			 * by others)
 			 */
 			CAlphabet* alpha=new CAlphabet(alphabet->get_alphabet());
 			SG_UNREF(alphabet);
@@ -216,6 +281,18 @@ template <class ST> class CStringFeatures : public CFeatures
 			SG_REF(alphabet);
 		}
 
+		/** cleanup a single feature vector */
+		virtual void cleanup_feature_vector(int32_t num)
+		{
+			ASSERT(num<num_vectors);
+			if (features)
+			{
+				delete[] features[num].string;
+				features[num].string=NULL;
+				features[num].length=0;
+			}
+		}
+
 		/** get feature class
 		 *
 		 * @return feature class STRING
@@ -262,9 +339,13 @@ template <class ST> class CStringFeatures : public CFeatures
 						"requested %d)\n", num_vectors, num);
 			}
 
-			*len=features[num].length;
+			int32_t l;
+			bool free_vec;
+			ST* vec=get_feature_vector(num, l, free_vec);
+			*len=l;
 			*dst=(ST*) malloc(*len * sizeof(ST));
-			memcpy(*dst, features[num].string, *len * sizeof(ST));
+			memcpy(*dst, vec, *len * sizeof(ST));
+			free_feature_vector(vec, num, free_vec);
 		}
 
 		/** set string for selected example num
@@ -286,7 +367,7 @@ template <class ST> class CStringFeatures : public CFeatures
 				SG_ERROR("String has zero or negative length\n");
 
 
-			delete[] features[num].string;
+			cleanup_feature_vector(num);
 			features[num].length=len;
 			features[num].string=new ST[len];
 			memcpy(features[num].string, src, len*sizeof(ST));
@@ -294,19 +375,77 @@ template <class ST> class CStringFeatures : public CFeatures
 			determine_maximum_string_length();
 		}
 
+		/** call this to preprocess string features upon get_feature_vector
+		 */
+		void enable_on_the_fly_preprocessing()
+		{
+			preprocess_on_get=true;
+		}
+
+		/** call this to disable on the fly feature preprocessing on
+		 * get_feature_vector. Useful when you manually apply preprocessors.
+		 */
+		void disable_on_the_fly_preprocessing()
+		{
+			preprocess_on_get=false;
+		}
+
 		/** get feature vector for sample num
 		 *
 		 * @param num index of feature vector
 		 * @param len length is returned by reference
+		 * @param dofree whether returned vector must be freed by
+		 * caller via free_feature_vector
 		 * @return feature vector for sample num
 		 */
-		virtual ST* get_feature_vector(int32_t num, int32_t& len)
+		ST* get_feature_vector(int32_t num, int32_t& len, bool& dofree)
 		{
 			ASSERT(features);
 			ASSERT(num<num_vectors);
 
-			len=features[num].length;
-			return features[num].string;
+			if (!preprocess_on_get)
+			{
+				dofree=false;
+				len=features[num].length;
+				return features[num].string;
+			}
+			else
+			{
+				SG_DEBUG( "computing feature vector!\n") ;
+				ST* feat=compute_feature_vector(num, len);
+				dofree=true;
+
+				if (get_num_preproc())
+				{
+					ST* tmp_feat_before = feat;
+
+					for (int32_t i=0; i<get_num_preproc(); i++)
+					{
+						CStringPreProc<ST>* p = (CStringPreProc<ST>*) get_preproc(i);
+						feat=p->apply_to_string(tmp_feat_before, len);
+						SG_UNREF(p);
+						delete[] tmp_feat_before;
+						tmp_feat_before=feat;
+					}
+				}
+				// TODO: implement caching
+				return feat;
+			}
+		}
+
+		/** free feature vector
+		 *
+		 * @param feat_vec feature vector to free
+		 * @param num index in feature cache
+		 * @param dofree if vector should be really deleted
+		 */
+		void free_feature_vector(ST* feat_vec, int32_t num, bool dofree)
+		{
+			if (feature_cache)
+				feature_cache->unlock_entry(num);
+
+			if (dofree)
+				delete[] feat_vec ;
 		}
 
 		/** get feature
@@ -317,10 +456,14 @@ template <class ST> class CStringFeatures : public CFeatures
 		 */
 		virtual ST inline get_feature(int32_t vec_num, int32_t feat_num)
 		{
-			ASSERT(features && vec_num<num_vectors);
-			ASSERT(feat_num<features[vec_num].length);
+			int32_t len;
+			bool free_vec;
+			ST* vec=get_feature_vector(vec_num, len, free_vec);
+			ASSERT(feat_num<len);
+			ST result=vec[feat_num];
+			free_feature_vector(vec, vec_num, free_vec);
 
-			return features[vec_num].string[feat_num];
+			return result;
 		}
 
 		/** get vector length
@@ -330,8 +473,11 @@ template <class ST> class CStringFeatures : public CFeatures
 		 */
 		virtual inline int32_t get_vector_length(int32_t vec_num)
 		{
-			ASSERT(features && vec_num<num_vectors);
-			return features[vec_num].length;
+			int32_t len;
+			bool free_vec;
+			ST* vec=get_feature_vector(vec_num, len, free_vec);
+			free_feature_vector(vec, vec_num, free_vec);
+			return len;
 		}
 
 		/** get maximum vector length
@@ -431,6 +577,9 @@ template <class ST> class CStringFeatures : public CFeatures
 			CFile f(fname, 'r', F_CHAR);
 			char* feature_matrix=f.load_char_data(NULL, length);
 
+			SG_DEBUG("char data now at %p of length %ld\n", 
+					feature_matrix, (int64_t) length);
+
 			num_vectors=0;
 
 			if (f.is_ok())
@@ -721,7 +870,7 @@ template <class ST> class CStringFeatures : public CFeatures
 				bool ignore_invalid=false, bool bitremap_in_single_string=false)
 		{
 			CMemoryMappedFile<char> f(fname);
-			
+
 			int32_t i=0;
 			uint64_t len=0;
 			uint64_t offs=0;
@@ -738,7 +887,7 @@ template <class ST> class CStringFeatures : public CFeatures
 			alphabet=new CAlphabet(DNA);
 
 			T_STRING<ST>* strings;
-			
+
 			ST* str;
 			if (bitremap_in_single_string)
 			{
@@ -844,7 +993,7 @@ template <class ST> class CStringFeatures : public CFeatures
 				int32_t num=0;
 				int32_t max_len=-1;
 
-				//usually n==num_vec, but it might not in race conditions 
+				//usually n==num_vec, but it might not in race conditions
 				//(file perms modified, file erased)
 				strings=new T_STRING<ST>[n];
 
@@ -930,7 +1079,7 @@ template <class ST> class CStringFeatures : public CFeatures
 			return false;
 		}
 
-		/** get_features 
+		/** get_features
 		 *
 		 * @param num_str number of strings (returned)
 		 * @param max_str_len maximal string length (returned)
@@ -943,6 +1092,34 @@ template <class ST> class CStringFeatures : public CFeatures
 			return features;
 		}
 
+		/** copy_features
+		 *
+		 * @param num_str number of strings (returned)
+		 * @param max_str_len maximal string length (returned)
+		 * @return string features
+		 */
+		virtual T_STRING<ST>* copy_features(int32_t& num_str, int32_t& max_str_len)
+		{
+			ASSERT(num_vectors>0);
+
+			num_str=num_vectors;
+			max_str_len=max_string_length;
+			T_STRING<ST>* new_feat=new T_STRING<ST>[num_str];
+
+			for (int i=0; i<num_str; i++)
+			{
+				int32_t len;
+				bool free_vec;
+				ST* vec=get_feature_vector(i, len, free_vec);
+				new_feat[i].string=new ST[len];
+				new_feat[i].length=len;
+				memcpy(new_feat[i].string, vec, ((size_t) len) * sizeof(ST));
+				free_feature_vector(vec, i, free_vec);
+			}
+
+			return new_feat;
+		}
+
 		/** get_features  (swig compatible)
 		 *
 		 * @param dst string features (returned)
@@ -950,8 +1127,10 @@ template <class ST> class CStringFeatures : public CFeatures
 		 */
 		virtual void get_features(T_STRING<ST>** dst, int32_t* num_str)
 		{
-			*num_str=num_vectors;
-			*dst=features;
+			int32_t num_vec;
+			int32_t max_str_len;
+			*dst=copy_features(num_vec, max_str_len);
+			*num_str=num_vec;
 		}
 
 		/** save features to file
@@ -964,6 +1143,157 @@ template <class ST> class CStringFeatures : public CFeatures
 			return false;
 		}
 
+		/** load compressed features from file
+		 *
+		 * @param src filename to load from
+		 * @param decompress whether to decompress on loading
+		 * @return if loading was successful
+		 */
+		virtual bool load_compressed(char* src, bool decompress)
+		{
+			FILE* file=NULL;
+
+			if (!(file=fopen(src, "r")))
+				return false;
+			cleanup();
+
+			// header shogun v0
+			char id[4];
+			fread(&id[0], sizeof(char), 1, file);
+			ASSERT(id[0]=='S');
+			fread(&id[1], sizeof(char), 1, file);
+			ASSERT(id[1]=='G');
+			fread(&id[2], sizeof(char), 1, file);
+			ASSERT(id[2]=='V');
+			fread(&id[3], sizeof(char), 1, file);
+			ASSERT(id[3]=='0');
+
+			//compression type
+			uint8_t c;
+			fread(&c, sizeof(uint8_t), 1, file);
+			CCompressor* compressor= new CCompressor((E_COMPRESSION_TYPE) c);
+			//alphabet
+			uint8_t a;
+			delete alphabet;
+			fread(&a, sizeof(uint8_t), 1, file);
+			alphabet=new CAlphabet((EAlphabet) a);
+			// number of vectors
+			fread(&num_vectors, sizeof(int32_t), 1, file);
+			ASSERT(num_vectors>0);
+			// maximum string length
+			fread(&max_string_length, sizeof(int32_t), 1, file);
+			ASSERT(max_string_length>0);
+
+			features=new T_STRING<ST>[num_vectors];
+
+			// vectors
+			for (int32_t i=0; i<num_vectors; i++)
+			{
+				// vector len compressed
+				int32_t len_compressed;
+				fread(&len_compressed, sizeof(int32_t), 1, file);
+				// vector len uncompressed
+				int32_t len_uncompressed;
+				fread(&len_uncompressed, sizeof(int32_t), 1, file);
+
+				// vector raw data
+				if (decompress)
+				{
+					features[i].string=new ST[len_uncompressed];
+					features[i].length=len_uncompressed;
+					uint8_t* compressed=new uint8_t[len_compressed];
+					fread(compressed, len_compressed, 1, file);
+					uint64_t uncompressed_size=len_uncompressed;
+					uncompressed_size*=sizeof(ST);
+					compressor->decompress(compressed, len_compressed,
+							(uint8_t*) features[i].string, uncompressed_size);
+					delete[] compressed;
+					ASSERT(uncompressed_size==((uint64_t) len_uncompressed)*sizeof(ST));
+				}
+				else
+				{
+					int32_t offs=CMath::ceil(2.0*sizeof(int32_t)/sizeof(ST));
+					features[i].string=new ST[len_compressed+offs];
+					features[i].length=len_compressed+offs;
+					int32_t* feat32ptr=((int32_t*) (features[i].string));
+					memset(features[i].string, 0, offs*sizeof(ST));
+					feat32ptr[0]=(int32_t) len_compressed;
+					feat32ptr[1]=(int32_t) len_uncompressed;
+					uint8_t* compressed=(uint8_t*) (&features[i].string[offs]);
+					fread(compressed, len_compressed, 1, file);
+				}
+			}
+
+			delete compressor;
+			fclose(file);
+			return false;
+		}
+
+		/** save compressed features to file
+		 *
+		 * @param dest filename to save to
+		 * @param compression compressor to use
+		 * @param level compression level to use (1-9)
+		 * @return if saving was successful
+		 */
+		virtual bool save_compressed(char* dest, E_COMPRESSION_TYPE compression, int level)
+		{
+			FILE* file=NULL;
+
+			if (!(file=fopen(dest, "wb")))
+				return false;
+
+			CCompressor* compressor= new CCompressor(compression);
+
+			// header shogun v0
+			const char* id="SGV0";
+			fwrite(&id[0], sizeof(char), 1, file);
+			fwrite(&id[1], sizeof(char), 1, file);
+			fwrite(&id[2], sizeof(char), 1, file);
+			fwrite(&id[3], sizeof(char), 1, file);
+
+			//compression type
+			uint8_t c=(uint8_t) compression;
+			fwrite(&c, sizeof(uint8_t), 1, file);
+			//alphabet
+			uint8_t a=(uint8_t) alphabet->get_alphabet();
+			fwrite(&a, sizeof(uint8_t), 1, file);
+			// number of vectors
+			fwrite(&num_vectors, sizeof(int32_t), 1, file);
+			// maximum string length
+			fwrite(&max_string_length, sizeof(int32_t), 1, file);
+
+			// vectors
+			for (int32_t i=0; i<num_vectors; i++)
+			{
+				int32_t len=-1;
+				bool vfree;
+				ST* vec=get_feature_vector(i, len, vfree);
+
+				uint8_t* compressed=NULL;
+				uint64_t compressed_size=0;
+
+				compressor->compress((uint8_t*) vec, ((uint64_t) len)*sizeof(ST),
+						compressed, compressed_size, level);
+
+				int32_t len_compressed = (int32_t) compressed_size;
+				// vector len compressed in bytes
+				fwrite(&len_compressed, sizeof(int32_t), 1, file);
+				// vector len uncompressed in number of elements of type ST
+				fwrite(&len, sizeof(int32_t), 1, file);
+				// vector raw data
+				fwrite(compressed, compressed_size, 1, file);
+				delete[] compressed;
+
+				free_feature_vector(vec, i, vfree);
+			}
+
+			delete compressor;
+			fclose(file);
+			return true;
+		}
+
+
 		/** get memory footprint of one feature
 		 *
 		 * @return memory footprint of one feature
@@ -980,7 +1310,7 @@ template <class ST> class CStringFeatures : public CFeatures
 			SG_DEBUG( "force: %d\n", force_preprocessing);
 
 			for (int32_t i=0; i<get_num_preproc(); i++)
-			{ 
+			{
 				if ( (!is_preprocessed(i) || force_preprocessing) )
 				{
 					set_preprocessed(i);
@@ -992,7 +1322,7 @@ template <class ST> class CStringFeatures : public CFeatures
 						SG_UNREF(p);
 						return false;
 					}
-					else 
+					else
 						SG_UNREF(p);
 				}
 			}
@@ -1151,7 +1481,9 @@ template <class ST> class CStringFeatures : public CFeatures
 				for (int32_t i=0; i<num_vectors; i++)
 				{
 					int32_t len=-1;
-					CT* c=sf->get_feature_vector(i, len);
+					bool vfree;
+					CT* c=sf->get_feature_vector(i, len, vfree);
+					ASSERT(!vfree); // won't work when preprocessors are attached
 
 					features[i].string=new ST[len];
 					features[i].length=len;
@@ -1159,7 +1491,6 @@ template <class ST> class CStringFeatures : public CFeatures
 					ST* str=features[i].string;
 					for (int32_t j=0; j<len; j++)
 						str[j]=(ST) alpha->remap_to_bin(c[j]);
-
 				}
 
 				original_num_symbols=alpha->get_num_symbols();
@@ -1183,18 +1514,20 @@ template <class ST> class CStringFeatures : public CFeatures
 				for (int32_t line=0; line<num_vectors; line++)
 				{
 					int32_t len=0;
-					ST* fv=get_feature_vector(line, len);
+					bool vfree;
+					ST* fv=get_feature_vector(line, len, vfree);
+					ASSERT(!vfree); // won't work when preprocessors are attached
 
 					if (rev)
-						translate_from_single_order_reversed(fv, len, start+gap, p_order+gap, max_val, gap);
+						CAlphabet::translate_from_single_order_reversed(fv, len, start+gap, p_order+gap, max_val, gap);
 					else
-						translate_from_single_order(fv, len, start+gap, p_order+gap, max_val, gap);
+						CAlphabet::translate_from_single_order(fv, len, start+gap, p_order+gap, max_val, gap);
 
 					/* fix the length of the string -- hacky */
 					features[line].length-=start+gap ;
 					if (features[line].length<0)
 						features[line].length=0 ;
-				}         
+				}
 
 				compute_symbol_mask_table(max_val);
 
@@ -1282,7 +1615,7 @@ template <class ST> class CStringFeatures : public CFeatures
 		}
 
 		/** compute symbol mask table
-		 * 
+		 *
 		 * required to access bit-based symbols
 		 */
 		inline void compute_symbol_mask_table(int64_t max_val)
@@ -1359,227 +1692,15 @@ template <class ST> class CStringFeatures : public CFeatures
 				max_string_length=CMath::max(max_string_length, features[i].length);
 		}
 
-		/** @return object name */
-		inline virtual const char* get_name() const { return "StringFeatures"; }
-
-	protected:
-
-		/** translate from single order
-		 *
-		 * @param obs observation
-		 * @param sequence_length length of sequence
-		 * @param start start
-		 * @param p_order order
-		 * @param max_val maximum value
-		 */
-		void translate_from_single_order(ST* obs, int32_t sequence_length, int32_t start, int32_t p_order, int32_t max_val)
+		static ST* get_zero_terminated_string_copy(T_STRING<ST> str)
 		{
-			int32_t i,j;
-			ST value=0;
-
-			for (i=sequence_length-1; i>= p_order-1; i--) //convert interval of size T
-			{
-				value=0;
-				for (j=i; j>=i-p_order+1; j--)
-					value= (value >> max_val) | (obs[j] << (max_val * (p_order-1)));
-
-				obs[i]= (ST) value;
-			}
-
-			for (i=p_order-2;i>=0;i--)
-			{
-				if (i>=sequence_length)
-					continue;
-
-				value=0;
-				for (j=i; j>=i-p_order+1; j--)
-				{
-					value= (value >> max_val);
-					if (j>=0 && j<sequence_length)
-						value|=obs[j] << (max_val * (p_order-1));
-				}
-				obs[i]=value;
-			}
-
-			// TODO we should get rid of this loop!
-			for (i=start; i<sequence_length; i++)
-				obs[i-start]=obs[i];
+			int32_t l=str.length;
+			ST* s=new ST[l+1];
+			memcpy(s, str.string, sizeof(ST)*l);
+			s[l]='\0';
+			return s;
 		}
 
-		/** translate from single order reversed
-		 *
-		 * @param obs observation
-		 * @param sequence_length length of sequence
-		 * @param start start
-		 * @param p_order order
-		 * @param max_val maximum value
-		 */
-		void translate_from_single_order_reversed(ST* obs, int32_t sequence_length, int32_t start, int32_t p_order, int32_t max_val)
-		{
-			int32_t i,j;
-			ST value=0;
-
-			for (i=sequence_length-1; i>= p_order-1; i--) //convert interval of size T
-			{
-				value=0;
-				for (j=i; j>=i-p_order+1; j--)
-					value= (value << max_val) | obs[j];
-
-				obs[i]= (ST) value;
-			}
-
-			for (i=p_order-2;i>=0;i--)
-			{
-				if (i>=sequence_length)
-					continue;
-
-				value=0;
-				for (j=i; j>=i-p_order+1; j--)
-				{
-					value= (value << max_val);
-					if (j>=0 && j<sequence_length)
-						value|=obs[j];
-				}
-				obs[i]=value;
-			}
-
-			// TODO we should get rid of this loop!
-			for (i=start; i<sequence_length; i++)
-				obs[i-start]=obs[i];
-		}
-
-		/** translate from single order
-		 *
-		 * @param obs observation
-		 * @param sequence_length length of sequence
-		 * @param start start
-		 * @param p_order order
-		 * @param max_val maximum value
-		 * @param gap gap
-		 */
-		void translate_from_single_order(ST* obs, int32_t sequence_length, int32_t start, int32_t p_order, int32_t max_val, int32_t gap)
-		{
-			ASSERT(gap>=0);
-
-			const int32_t start_gap=(p_order-gap)/2;
-			const int32_t end_gap=start_gap+gap;
-
-			int32_t i,j;
-			ST value=0;
-
-			// almost all positions
-			for (i=sequence_length-1; i>=p_order-1; i--) //convert interval of size T
-			{
-				value=0;
-				for (j=i; j>=i-p_order+1; j--)
-				{
-					if (i-j<start_gap)
-					{
-						value= (value >> max_val) | (obs[j] << (max_val * (p_order-1-gap)));
-					}
-					else if (i-j>=end_gap)
-					{
-						value= (value >> max_val) | (obs[j] << (max_val * (p_order-1-gap)));
-					}
-				}
-				obs[i]= (ST) value;
-			}
-
-			// the remaining `order` positions
-			for (i=p_order-2;i>=0;i--)
-			{
-				if (i>=sequence_length)
-					continue;
-
-				value=0;
-				for (j=i; j>=i-p_order+1; j--)
-				{
-					if (i-j<start_gap)
-					{
-						value= (value >> max_val);
-						if (j>=0 && j<sequence_length)
-							value|=obs[j] << (max_val * (p_order-1-gap));
-					}
-					else if (i-j>=end_gap)
-					{
-						value= (value >> max_val);
-						if (j>=0 && j<sequence_length)
-							value|=obs[j] << (max_val * (p_order-1-gap));
-					}
-				}
-				obs[i]=value;
-			}
-
-			// TODO we should get rid of this loop!
-			for (i=start; i<sequence_length; i++)
-				obs[i-start]=obs[i];
-		}
-
-		/** translate from single order reversed
-		 *
-		 * @param obs observation
-		 * @param sequence_length length of sequence
-		 * @param start start
-		 * @param p_order order
-		 * @param max_val maximum value
-		 * @param gap gap
-		 */
-		void translate_from_single_order_reversed(ST* obs, int32_t sequence_length, int32_t start, int32_t p_order, int32_t max_val, int32_t gap)
-		{
-			ASSERT(gap>=0);
-
-			const int32_t start_gap=(p_order-gap)/2;
-			const int32_t end_gap=start_gap+gap;
-
-			int32_t i,j;
-			ST value=0;
-
-			// almost all positions
-			for (i=sequence_length-1; i>=p_order-1; i--) //convert interval of size T
-			{
-				value=0;
-				for (j=i; j>=i-p_order+1; j--)
-				{
-					if (i-j<start_gap)
-						value= (value << max_val) | obs[j];
-					else if (i-j>=end_gap)
-						value= (value << max_val) | obs[j];
-				}
-				obs[i]= (ST) value;
-			}
-
-			// the remaining `order` positions
-			for (i=p_order-2;i>=0;i--)
-			{
-				if (i>=sequence_length)
-					continue;
-
-				value=0;
-				for (j=i; j>=i-p_order+1; j--)
-				{
-					if (i-j<start_gap)
-					{
-						value= value << max_val;
-						if (j>=0 && j<sequence_length)
-							value|=obs[j];
-					}
-					else if (i-j>=end_gap)
-					{
-						value= value << max_val;
-						if (j>=0 && j<sequence_length)
-							value|=obs[j];
-					}			
-				}
-				obs[i]=value;
-			}
-
-			// TODO we should get rid of this loop!
-			for (i=start; i<sequence_length; i++)
-				obs[i-start]=obs[i];
-		}
-
-	protected:
-
 		/** set feature vector for sample num
 		 *
 		 * @param num index of feature vector
@@ -1593,9 +1714,123 @@ template <class ST> class CStringFeatures : public CFeatures
 
 			features[num].length=len ;
 			features[num].string=string ;
+
+			max_string_length=CMath::max(len, max_string_length);
 		}
 
 
+		/** @return object name */
+		inline virtual const char* get_name() const { return "StringFeatures"; }
+
+	protected:
+
+		/** compute feature vector for sample num
+		 * if target is set the vector is written to target
+		 * len is returned by reference
+		 *
+		 * default implementation returns
+		 *
+		 * @param num which vector
+		 * @param len length of vector
+		 * @return feature vector
+		 */
+		virtual ST* compute_feature_vector(int32_t num, int32_t& len)
+		{
+			ASSERT(features && num<num_vectors);
+
+			len=features[num].length;
+			if (len<=0)
+				return NULL;
+
+			ST* target=new ST[len];
+			memcpy(target, features[num].string, len*sizeof(ST));
+			return target;
+		}
+
+#ifdef HAVE_BOOST_SERIALIZATION
+    private:
+
+        friend class ::boost::serialization::access;
+        template<class Archive>
+            void save(Archive & ar, const unsigned int archive_version) const
+            {
+
+				SG_DEBUG("archiving StringFeatures\n");
+
+                ar & ::boost::serialization::base_object<CFeatures>(*this);
+
+                ar & alphabet;
+
+                ar & num_vectors;
+                for (int i=0; i < num_vectors; ++i) {
+                    ar & features[i];
+                }
+
+                ar & length_of_single_string;
+                for (int i=0; i < length_of_single_string; ++i) {
+                    ar & single_string[i];
+                }
+
+                ar & max_string_length;
+                ar & num_symbols;
+                ar & original_num_symbols;
+                ar & order;
+
+                /// order used in higher order mapping
+                //TODO?! how long
+                //ST* symbol_mask_table;
+
+                SG_DEBUG("done archiving StringFeatures\n");
+
+            }
+
+        template<class Archive>
+            void load(Archive & ar, const unsigned int archive_version)
+            {
+
+				SG_DEBUG("archiving StringFeatures\n");
+
+                ar & ::boost::serialization::base_object<CFeatures>(*this);
+
+
+                ar & alphabet;
+
+                ar & num_vectors;
+
+                //T_STRING<ST>* features = new T_STRING<ST>[num_vectors];
+                features = new T_STRING<ST>[num_vectors];
+                for (int i=0; i < num_vectors; ++i) {
+                    ar & features[i];
+                }
+
+
+                ar & length_of_single_string;
+
+                //ST* single_string = new ST[length_of_single_string];
+                single_string = new ST[length_of_single_string];
+                for (int i=0; i < length_of_single_string; ++i) {
+                    ar & single_string[i];
+                }
+
+                ar & max_string_length;
+                ar & num_symbols;
+                ar & original_num_symbols;
+                ar & order;
+
+                /// order used in higher order mapping
+                //TODO?! how long -> num_of_symbols?
+                //ST* symbol_mask_table;
+
+                SG_DEBUG("done archiving StringFeatures\n");
+
+            }
+
+        GLOBAL_BOOST_SERIALIZATION_SPLIT_MEMBER();
+
+
+#endif //HAVE_BOOST_SERIALIZATION
+
+
 	protected:
 
 		/// alphabet
@@ -1627,6 +1862,12 @@ template <class ST> class CStringFeatures : public CFeatures
 
 		/// order used in higher order mapping
 		ST* symbol_mask_table;
+
+		/// preprocess on-the-fly?
+		bool preprocess_on_get;
+
+		/** feature cache */
+		CCache<ST>* feature_cache;
 };
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
@@ -1789,30 +2030,7 @@ template<> inline floatmax_t CStringFeatures<floatmax_t>::shift_symbol(floatmax_
 	return symbol;
 }
 
-template<> inline void CStringFeatures<float32_t>::translate_from_single_order(float32_t* obs, int32_t sequence_length, int32_t start, int32_t p_order, int32_t max_val, int32_t gap)
-{
-}
-
-template<> inline void CStringFeatures<float64_t>::translate_from_single_order(float64_t* obs, int32_t sequence_length, int32_t start, int32_t p_order, int32_t max_val, int32_t gap)
-{
-}
-
-template<> inline void CStringFeatures<floatmax_t>::translate_from_single_order(floatmax_t* obs, int32_t sequence_length, int32_t start, int32_t p_order, int32_t max_val, int32_t gap)
-{
-}
-
-template<> inline void CStringFeatures<float32_t>::translate_from_single_order_reversed(float32_t* obs, int32_t sequence_length, int32_t start, int32_t p_order, int32_t max_val, int32_t gap)
-{
-}
-
-template<> inline void CStringFeatures<float64_t>::translate_from_single_order_reversed(float64_t* obs, int32_t sequence_length, int32_t start, int32_t p_order, int32_t max_val, int32_t gap)
-{
-}
-
-template<> inline void CStringFeatures<floatmax_t>::translate_from_single_order_reversed(floatmax_t* obs, int32_t sequence_length, int32_t start, int32_t p_order, int32_t max_val, int32_t gap)
-{
-}
-
+#ifndef SUNOS
 template<> 	template <class CT> bool CStringFeatures<float32_t>::obtain_from_char_features(CStringFeatures<CT>* sf, int32_t start, int32_t p_order, int32_t gap, bool rev)
 {
 	return false;
@@ -1825,6 +2043,7 @@ template<> 	template <class CT> bool CStringFeatures<floatmax_t>::obtain_from_ch
 {
 	return false;
 }
+#endif
 
 template<> 	inline void CStringFeatures<float32_t>::embed_features(int32_t p_order)
 {
@@ -1869,4 +2088,5 @@ template<> 	inline void CStringFeatures<floatmax_t>::unembed_word(floatmax_t wor
 {
 }
 #endif // DOXYGEN_SHOULD_SKIP_THIS
+}
 #endif // _CSTRINGFEATURES__H__
