@@ -25,6 +25,8 @@
 #include <pthread.h>
 #endif
 
+using namespace shogun;
+
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 struct S_THREAD_PARAM
 {
@@ -43,10 +45,12 @@ struct S_THREAD_PARAM
 CCombinedKernel::CCombinedKernel(int32_t size, bool asw)
 : CKernel(size), sv_count(0), sv_idx(NULL), sv_weight(NULL),
 	subkernel_weights_buffer(NULL), append_subkernel_weights(asw),
-	num_lhs(0), num_rhs(0), initialized(false)
+	initialized(false)
 {
 	properties |= KP_LINADD | KP_KERNCOMBINATION | KP_BATCHEVALUATION;
 	kernel_list=new CList<CKernel*>(true);
+	SG_REF(kernel_list);
+
 	SG_INFO("Combined kernel created (%p)\n", this) ;
 	if (append_subkernel_weights)
 		SG_INFO( "(subkernel weights are appended)\n") ;
@@ -71,12 +75,6 @@ bool CCombinedKernel::init(CFeatures* l, CFeatures* r)
 	ASSERT(l->get_feature_type()==F_UNKNOWN);
 	ASSERT(r->get_feature_type()==F_UNKNOWN);
 
-	ASSERT(!num_lhs || num_lhs==l->get_num_vectors());
-	ASSERT(!num_rhs || num_rhs==l->get_num_vectors());
-
-	num_lhs=l->get_num_vectors();
-	num_rhs=r->get_num_vectors();
-
 	CFeatures* lf=NULL;
 	CFeatures* rf=NULL;
 	CKernel* k=NULL;
@@ -116,9 +114,9 @@ bool CCombinedKernel::init(CFeatures* l, CFeatures* r)
 			SG_DEBUG( "Initializing 0x%p - \"%s\" (skipping init, this is a CUSTOM kernel)\n", this, k->get_name());
 			if (!k->has_features())
 				SG_ERROR("No kernel matrix was assigned to this Custom kernel\n");
-			if (!k->get_num_vec_lhs() != num_lhs)
+			if (k->get_num_vec_lhs() != num_lhs)
 				SG_ERROR("Number of lhs-feature vectors (%d) not match with number of rows (%d) of custom kernel\n", num_lhs, k->get_num_vec_lhs());
-			if (!k->get_num_vec_rhs() != num_rhs)
+			if (k->get_num_vec_rhs() != num_rhs)
 				SG_ERROR("Number of rhs-feature vectors (%d) not match with number of cols (%d) of custom kernel\n", num_rhs, k->get_num_vec_rhs());
 		}
 
@@ -769,6 +767,7 @@ bool CCombinedKernel::precompute_subkernels()
 
 	SG_UNREF(kernel_list);
 	kernel_list=new_kernel_list;
+	SG_REF(kernel_list);
 
 	return true;
 }
