@@ -14,6 +14,8 @@
 #include "kernel/DiagKernel.h"
 #include "lib/io.h"
 
+using namespace shogun;
+
 CDiagKernel::CDiagKernel(int32_t size, float64_t d)
 : CKernel(size), diag(d)
 {
@@ -28,14 +30,3 @@ CDiagKernel::CDiagKernel(CFeatures* l, CFeatures* r, float64_t d)
 CDiagKernel::~CDiagKernel()
 {
 }
-
-bool CDiagKernel::load_init(FILE* src)
-{
-	return false;
-}
-
-bool CDiagKernel::save_init(FILE* dest)
-{
-	return false;
-}
-
