@@ -24,9 +24,18 @@
 #include <pthread.h>
 #endif
 
+
+#ifdef HAVE_BOOST_SERIALIZATION
+#include <boost/serialization/export.hpp>
+BOOST_CLASS_EXPORT(shogun::CWeightedDegreeStringKernel);
+#endif //HAVE_BOOST_SERIALIZATION
+
+using namespace shogun;
+
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 struct S_THREAD_PARAM
 {
+
 	int32_t* vec;
 	float64_t* result;
 	float64_t* weights;
@@ -41,6 +50,7 @@ struct S_THREAD_PARAM
 };
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 
+
 CWeightedDegreeStringKernel::CWeightedDegreeStringKernel (
 	int32_t degree_, EWDKernType type_)
 : CStringKernel<char>(10),weights(NULL),position_weights(NULL),
@@ -114,7 +124,7 @@ CWeightedDegreeStringKernel::~CWeightedDegreeStringKernel()
 
 
 void CWeightedDegreeStringKernel::remove_lhs()
-{ 
+{
 	SG_DEBUG( "deleting CWeightedDegreeStringKernel optimization\n");
 	delete_optimization();
 
@@ -126,6 +136,8 @@ void CWeightedDegreeStringKernel::remove_lhs()
 
 void CWeightedDegreeStringKernel::create_empty_tries()
 {
+	ASSERT(lhs);
+
 	seq_length=((CStringFeatures<char>*) lhs)->get_max_vector_length();
 
 	if (tries!=NULL)
@@ -134,7 +146,7 @@ void CWeightedDegreeStringKernel::create_empty_tries()
 		tries->create(seq_length, max_mismatch==0) ;
 	}
 }
-  
+
 bool CWeightedDegreeStringKernel::init(CFeatures* l, CFeatures* r)
 {
 	int32_t lhs_changed=(lhs!=l);
@@ -158,7 +170,7 @@ bool CWeightedDegreeStringKernel::init(CFeatures* l, CFeatures* r)
 	SG_UNREF(alphabet);
 	alphabet=sf_l->get_alphabet();
 	CAlphabet* ralphabet=sf_r->get_alphabet();
-	
+
 	if (!((alphabet->get_alphabet()==DNA) || (alphabet->get_alphabet()==RNA)))
 		properties &= ((uint64_t) (-1)) ^ (KP_LINADD | KP_BATCHEVALUATION);
 
@@ -201,17 +213,6 @@ void CWeightedDegreeStringKernel::cleanup()
 	CKernel::cleanup();
 }
 
-bool CWeightedDegreeStringKernel::load_init(FILE* src)
-{
-	return false;
-}
-
-bool CWeightedDegreeStringKernel::save_init(FILE* dest)
-{
-	return false;
-}
-  
-
 bool CWeightedDegreeStringKernel::init_optimization(int32_t count, int32_t* IDX, float64_t* alphas, int32_t tree_num)
 {
 	if (tree_num<0)
@@ -228,7 +229,7 @@ bool CWeightedDegreeStringKernel::init_optimization(int32_t count, int32_t* IDX,
 		{
 			if ( (i % (count/10+1)) == 0)
 				SG_PROGRESS(i, 0, count);
-			
+
 			if (max_mismatch==0)
 				add_example_to_tree(IDX[i], alphas[i]) ;
 			else
@@ -244,7 +245,7 @@ bool CWeightedDegreeStringKernel::init_optimization(int32_t count, int32_t* IDX,
 				add_example_to_single_tree_mismatch(IDX[i], alphas[i], tree_num) ;
 		}
 	}
-	
+
 	if (tree_num<0)
 		SG_DONE();
 
@@ -255,15 +256,15 @@ bool CWeightedDegreeStringKernel::init_optimization(int32_t count, int32_t* IDX,
 }
 
 bool CWeightedDegreeStringKernel::delete_optimization()
-{ 
+{
 	if (get_is_initialized())
 	{
 		if (tries!=NULL)
-			tries->delete_trees(max_mismatch==0); 
+			tries->delete_trees(max_mismatch==0);
 		set_is_initialized(false);
 		return true;
 	}
-	
+
 	return false;
 }
 
@@ -372,8 +373,9 @@ float64_t CWeightedDegreeStringKernel::compute_without_mismatch_matrix(
 float64_t CWeightedDegreeStringKernel::compute(int32_t idx_a, int32_t idx_b)
 {
 	int32_t alen, blen;
-	char* avec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx_a, alen);
-	char* bvec=((CStringFeatures<char>*) rhs)->get_feature_vector(idx_b, blen);
+	bool free_avec, free_bvec;
+	char* avec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx_a, alen, free_avec);
+	char* bvec=((CStringFeatures<char>*) rhs)->get_feature_vector(idx_b, blen, free_bvec);
 	float64_t result=0;
 
 	if (max_mismatch==0 && length==0 && block_computation)
@@ -387,6 +389,8 @@ float64_t CWeightedDegreeStringKernel::compute(int32_t idx_a, int32_t idx_b)
 		else
 			result=compute_without_mismatch_matrix(avec, alen, bvec, blen);
 	}
+	((CStringFeatures<char>*) lhs)->free_feature_vector(avec, idx_a, free_avec);
+	((CStringFeatures<char>*) rhs)->free_feature_vector(bvec, idx_b, free_bvec);
 
 	return result;
 }
@@ -399,13 +403,15 @@ void CWeightedDegreeStringKernel::add_example_to_tree(
 	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
 	int32_t len=0;
-	char* char_vec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx, len);
+	bool free_vec;
+	char* char_vec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx, len, free_vec);
 	ASSERT(max_mismatch==0);
 	int32_t *vec=new int32_t[len];
 
 	for (int32_t i=0; i<len; i++)
 		vec[i]=alphabet->remap_to_bin(char_vec[i]);
-	
+	((CStringFeatures<char>*) lhs)->free_feature_vector(char_vec, idx, free_vec);
+
 	if (length == 0 || max_mismatch > 0)
 	{
 		for (int32_t i=0; i<len; i++)
@@ -424,7 +430,7 @@ void CWeightedDegreeStringKernel::add_example_to_tree(
 		for (int32_t i=0; i<len; i++)
 		{
 			float64_t alpha_pw=alpha;
-			/*if (position_weights!=NULL) 
+			/*if (position_weights!=NULL)
 			  alpha_pw = alpha*position_weights[i] ;*/
 			if (alpha_pw==0.0)
 				continue ;
@@ -437,19 +443,22 @@ void CWeightedDegreeStringKernel::add_example_to_tree(
 }
 
 void CWeightedDegreeStringKernel::add_example_to_single_tree(
-	int32_t idx, float64_t alpha, int32_t tree_num) 
+	int32_t idx, float64_t alpha, int32_t tree_num)
 {
 	ASSERT(alphabet);
 	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
-	int32_t len ;
-	char* char_vec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx, len);
+	int32_t len;
+	bool free_vec;
+	char* char_vec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx, len, free_vec);
 	ASSERT(max_mismatch==0);
 	int32_t *vec = new int32_t[len] ;
 
 	for (int32_t i=tree_num; i<tree_num+degree && i<len; i++)
 		vec[i]=alphabet->remap_to_bin(char_vec[i]);
-	
+	((CStringFeatures<char>*) lhs)->free_feature_vector(char_vec, idx, free_vec);
+
+
 	ASSERT(tries);
 	if (alpha!=0.0)
 		tries->add_to_trie(tree_num, 0, vec, normalizer->normalize_lhs(alpha, idx), weights, (length!=0));
@@ -465,19 +474,21 @@ void CWeightedDegreeStringKernel::add_example_to_tree_mismatch(int32_t idx, floa
 	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
 	int32_t len ;
-	char* char_vec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx, len);
-	
+	bool free_vec;
+	char* char_vec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx, len, free_vec);
+
 	int32_t *vec = new int32_t[len] ;
-	
+
 	for (int32_t i=0; i<len; i++)
 		vec[i]=alphabet->remap_to_bin(char_vec[i]);
-	
+	((CStringFeatures<char>*) lhs)->free_feature_vector(char_vec, idx, free_vec);
+
 	for (int32_t i=0; i<len; i++)
 	{
 		if (alpha!=0.0)
 			tries->add_example_to_tree_mismatch_recursion(NO_CHILD, i, normalizer->normalize_lhs(alpha, idx), &vec[i], len-i, 0, 0, max_mismatch, weights);
 	}
-	
+
 	delete[] vec ;
 	tree_initialized=true ;
 }
@@ -490,11 +501,13 @@ void CWeightedDegreeStringKernel::add_example_to_single_tree_mismatch(
 	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
 	int32_t len=0;
-	char* char_vec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx, len);
+	bool free_vec;
+	char* char_vec=((CStringFeatures<char>*) lhs)->get_feature_vector(idx, len, free_vec);
 	int32_t *vec=new int32_t[len];
 
 	for (int32_t i=tree_num; i<len && i<tree_num+degree; i++)
 		vec[i]=alphabet->remap_to_bin(char_vec[i]);
+	((CStringFeatures<char>*) lhs)->free_feature_vector(char_vec, idx, free_vec);
 
 	if (alpha!=0.0)
 	{
@@ -514,12 +527,14 @@ float64_t CWeightedDegreeStringKernel::compute_by_tree(int32_t idx)
 	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
 	int32_t len=0;
-	char* char_vec=((CStringFeatures<char>*) rhs)->get_feature_vector(idx, len);
+	bool free_vec;
+	char* char_vec=((CStringFeatures<char>*) rhs)->get_feature_vector(idx, len, free_vec);
 	ASSERT(char_vec && len>0);
 	int32_t *vec=new int32_t[len];
 
 	for (int32_t i=0; i<len; i++)
 		vec[i]=alphabet->remap_to_bin(char_vec[i]);
+	((CStringFeatures<char>*) lhs)->free_feature_vector(char_vec, idx, free_vec);
 
 	float64_t sum=0;
 	ASSERT(tries);
@@ -537,12 +552,14 @@ void CWeightedDegreeStringKernel::compute_by_tree(
 	ASSERT(alphabet->get_alphabet()==DNA || alphabet->get_alphabet()==RNA);
 
 	int32_t len ;
-	char* char_vec=((CStringFeatures<char>*) rhs)->get_feature_vector(idx, len);
+	bool free_vec;
+	char* char_vec=((CStringFeatures<char>*) rhs)->get_feature_vector(idx, len, free_vec);
 
 	int32_t *vec = new int32_t[len] ;
 
 	for (int32_t i=0; i<len; i++)
 		vec[i]=alphabet->remap_to_bin(char_vec[i]);
+	((CStringFeatures<char>*) lhs)->free_feature_vector(char_vec, idx, free_vec);
 
 	ASSERT(tries);
 	for (int32_t i=0; i<len; i++)
@@ -614,16 +631,18 @@ bool CWeightedDegreeStringKernel::set_wd_weights_by_type(EWDKernType p_type)
 bool CWeightedDegreeStringKernel::set_weights(
 	float64_t* ws, int32_t d, int32_t len)
 {
-	SG_DEBUG("degree = %i  d=%i\n", degree, d);
-	degree=d;
-	ASSERT(tries);
-	tries->set_degree(degree);
+	if (d!=degree || len<1)
+		SG_ERROR("Dimension mismatch (should be de(seq_length | 1) x degree)\n");
+
 	length=len;
-	
-	if (length==0) length=1;
+
+	if (length==0)
+		length=1;
+
 	int32_t num_weights=degree*(length+max_mismatch);
 	delete[] weights;
 	weights=new float64_t[num_weights];
+
 	if (weights)
 	{
 		for (int32_t i=0; i<num_weights; i++) {
@@ -654,7 +673,7 @@ bool CWeightedDegreeStringKernel::set_position_weights(
 	position_weights=new float64_t[len];
 	ASSERT(tries);
 	tries->set_position_weights(position_weights);
-	
+
 	if (position_weights)
 	{
 		for (int32_t i=0; i<len; i++)
@@ -875,9 +894,11 @@ void* CWeightedDegreeStringKernel::compute_batch_helper(void* p)
 	for (int32_t i=params->start; i<params->end; i++)
 	{
 		int32_t len=0;
-		char* char_vec=rhs_feat->get_feature_vector(vec_idx[i], len);
+		bool free_vec;
+		char* char_vec=rhs_feat->get_feature_vector(vec_idx[i], len, free_vec);
 		for (int32_t k=j; k<CMath::min(len,j+wd->get_degree()); k++)
 			vec[k]=alpha->remap_to_bin(char_vec[k]);
+		rhs_feat->free_feature_vector(char_vec, vec_idx[i], free_vec);
 
 		ASSERT(tries);
 
