@@ -28,6 +28,10 @@
 #include <sys/time.h>
 #include <time.h>
 
+#ifdef SUNOS
+#include <ieeefp.h>
+#endif
+
 /// workaround a bug in std cmath
 #ifdef _GLIBCXX_CMATH
 #if _GLIBCXX_USE_C99_MATH
@@ -112,8 +116,8 @@ template <class T1, class T2> struct thread_qsort
 };
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 
-class CSGObject;
-
+namespace shogun
+{
 /** @brief Class which collects generic mathematical functions
  */
 class CMath : public CSGObject
@@ -138,14 +142,14 @@ class CMath : public CSGObject
 		template <class T>
 			static inline T min(T a, T b)
 			{
-				return ((a)<=(b))?(a):(b);
+				return (a<=b) ? a : b;
 			}
 
 		///return the maximum of two integers
 		template <class T>
 			static inline T max(T a, T b) 
 			{
-				return ((a)>=(b))?(a):(b);
+				return (a>=b) ? a : b;
 			}
 
 		///return the value clamped to interval [lb,ub]
@@ -178,9 +182,6 @@ class CMath : public CSGObject
 		/**@name misc functions */
 		//@{
 
-		/// crc32
-		static uint32_t crc32(uint8_t *data, int32_t len);
-
 		static inline float64_t round(float64_t d)
 		{
 			return ::floor(d+0.5);
@@ -214,6 +215,32 @@ class CMath : public CSGObject
 				b=c;
 			}
 
+		/** resize array from old_size to new_size (keeping as much array
+		 * content as possible intact)
+		 */
+		template <class T>
+			static inline void resize(T* &data, int64_t old_size, int64_t new_size)
+			{
+				if (old_size==new_size)
+					return;
+				T* new_data = new T[new_size];
+				for (int64_t i=0; i<old_size && i<new_size; i++)
+					new_data[i]=data[i];
+				delete[] data;
+				data=new_data;
+			}
+
+		/// || x ||_2
+		template <class T>
+			static inline T twonorm(T* x, int32_t len)
+			{
+				float64_t result=0;
+				for (int32_t i=0; i<len; i++)
+					result+=x[i]*x[i];
+
+				return CMath::sqrt(result);
+			}
+
 		/// || x ||_q^q
 		template <class T>
 			static inline T qsq(T* x, int32_t len, float64_t q)
@@ -935,11 +962,11 @@ class CMath : public CSGObject
 
 		/// display vector (useful for debugging)
 		template <class T> static void display_vector(
-			T* vector, int32_t n, const char* name="vector");
+			const T* vector, int32_t n, const char* name="vector");
 
 		/// display matrix (useful for debugging)
 		template <class T> static void display_matrix(
-			T* matrix, int32_t rows, int32_t cols, const char* name="matrix");
+			const T* matrix, int32_t rows, int32_t cols, const char* name="matrix");
 
 		/** performs a quicksort on an array output of length size
 		 * it is sorted in ascending order 
@@ -1017,7 +1044,7 @@ class CMath : public CSGObject
 				int32_t end=size-1;
 
 				if (size<1)
-					return -1;
+					return 0;
 
 				while (start<end)
 				{
@@ -1031,10 +1058,7 @@ class CMath : public CSGObject
 						return middle;
 				}
 
-				if (output[start]==elem)
-					return start;
-				else
-					return -1;
+				return start;
 			}
 
 		/* finds an element in a sorted array via binary search
@@ -1101,8 +1125,8 @@ class CMath : public CSGObject
 		/// checks whether a float is finite
 		inline static int is_finite(double f)
 		{
-#ifdef isfinite
-            return isfinite(f);
+#if defined(isfinite) && !defined(SUNOS)
+			return isfinite(f);
 #else
 			return finite(f);
 #endif
@@ -1111,13 +1135,24 @@ class CMath : public CSGObject
 		/// checks whether a float is infinity
 		inline static int is_infinity(double f)
 		{
+#ifdef SUNOS
+			if (fpclass(f) == FP_NINF || fpclass(f) == FP_PINF)
+				return 1;
+			else
+				return 0;
+#else
             return isinf(f);
+#endif
 		}
 
 		/// checks whether a float is nan
 		inline static int is_nan(double f)
 		{
+#ifdef SUNOS
+			return isnand(f);
+#else
             return isnan(f);
+#endif
 		}
 
 
@@ -1450,4 +1485,5 @@ void CMath::min(float64_t* output, T* index, int32_t size)
 	swap(output[0], output[min_index]);
 	swap(index[0], index[min_index]);
 }
+}
 #endif
