@@ -12,19 +12,14 @@
  * Copyright (C) 1999-2009 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
+#include "lib/memory.h"
+
 #ifndef __COMMON_H__
 #define __COMMON_H__
 
 #include <stdlib.h>
 #include <stdio.h>
 #include "lib/config.h"
-#include "lib/memory.h"
-
-#ifdef SUNOS
-#define bool int
-#define false 0
-#define true 1
-#endif
 
 #ifndef LINUX
 #define RANDOM_MAX 2147483647
@@ -32,25 +27,12 @@
 #define RANDOM_MAX RAND_MAX
 #endif
 
-/**@name Standard Types 
+/**@name Standard Types
  * Definition of Platform independent Types
 */
 //@{
 
-#ifdef SUNOS
-typedef char int8_t;
-typedef short int int16_t;
-typedef int int32_t;
-typedef long int64_t;
-typedef unsigned char uint8_t;
-typedef unsigned short int uint16_t;
-typedef unsigned int uint32_t;
-typedef unsigned long uint64_t;
-typedef unsigned long long int uintmax_t;
-#else
 #include <stdint.h>
-#endif
-
 
 /**
  * Implementations tend to follow IEEE754
@@ -62,4 +44,15 @@ typedef long double floatmax_t;
 
 //@}
 
-#endif
+#ifdef HAVE_BOOST_SERIALIZATION
+// split member function serialize funcition into save/load
+#define GLOBAL_BOOST_SERIALIZATION_SPLIT_MEMBER()                  \
+template<class Archive>                                            \
+void serialize(Archive &ar, const unsigned int file_version)       \
+{                                                                  \
+    ::boost::serialization::split_member(ar, *this, file_version); \
+}
+/**/
+#endif //HAVE_BOOST_SERIALIZATION
+
+#endif //__COMMON_H__
