@@ -4,9 +4,9 @@
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2008 Soeren Sonnenburg
+ * Written (W) 1999-2009 Soeren Sonnenburg
  * Written (W) 1999-2008 Gunnar Raetsch
- * Copyright (C) 1999-2008 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Copyright (C) 1999-2009 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 #include "GUIClassifier.h"
 #include "SGInterface.h"
@@ -35,7 +35,12 @@
 
 
 
+#include <shogun/classifier/mkl/MKLClassification.h>
+#include <shogun/regression/svr/MKLRegression.h>
+#include <shogun/classifier/mkl/MKLOneClass.h>
+#include <shogun/classifier/mkl/MKLMultiClass.h>
 #include <shogun/classifier/svm/LibSVM.h>
+#include <shogun/classifier/svm/LaRank.h>
 #include <shogun/classifier/svm/GPBTSVM.h>
 #include <shogun/classifier/svm/LibSVMOneClass.h>
 #include <shogun/classifier/svm/LibSVMMultiClass.h>
@@ -47,6 +52,7 @@
 #include <shogun/classifier/svm/MPDSVM.h>
 #include <shogun/classifier/svm/GNPPSVM.h>
 #include <shogun/classifier/svm/GMNPSVM.h>
+#include <shogun/classifier/svm/ScatterSVM.h>
 
 #include <shogun/classifier/svm/SVMLin.h>
 #include <shogun/classifier/svm/SubGradientSVM.h>
@@ -55,9 +61,12 @@
 #include <shogun/classifier/svm/SVMSGD.h>
 #include <shogun/classifier/svm/WDSVMOcas.h>
 
+using namespace shogun;
+
 CGUIClassifier::CGUIClassifier(CSGInterface* ui_)
 : CSGObject(), ui(ui_)
 {
+	constraint_generator=NULL;
 	classifier=NULL;
 	max_train_time=0;
 
@@ -69,10 +78,11 @@ CGUIClassifier::CGUIClassifier(CSGInterface* ui_)
 	svm_qpsize=41;
 	svm_bufsize=3000;
 	svm_max_qpsize=1000;
-	svm_mkl_norm=1;
+	mkl_norm=1;
 	svm_C1=1;
 	svm_C2=1;
-	svm_C_mkl=0;
+	C_mkl=0;
+	mkl_use_interleaved=true;
 	svm_weight_epsilon=1e-5;
 	svm_epsilon=1e-5;
 	svm_tube_epsilon=1e-2;
@@ -80,7 +90,6 @@ CGUIClassifier::CGUIClassifier(CSGInterface* ui_)
 	svm_use_shrinking = true ;
 
 	svm_use_bias = true;
-	svm_use_mkl = false ;
 	svm_use_batch_computation = true ;
 	svm_use_linadd = true ;
 	svm_do_auc_maximization = false ;
@@ -94,6 +103,7 @@ CGUIClassifier::CGUIClassifier(CSGInterface* ui_)
 CGUIClassifier::~CGUIClassifier()
 {
 	SG_UNREF(classifier);
+	SG_UNREF(constraint_generator);
 }
 
 bool CGUIClassifier::new_classifier(char* name, int32_t d, int32_t from_d)
@@ -110,12 +120,36 @@ bool CGUIClassifier::new_classifier(char* name, int32_t d, int32_t from_d)
 		classifier = new CLibSVMMultiClass();
 		SG_INFO("created SVMlibsvm object for multiclass\n");
 	}
+	else if (strcmp(name,"LIBSVM_NUMULTICLASS")==0)
+	{
+		SG_UNREF(classifier);
+		classifier= new CLibSVMMultiClass(LIBSVM_NU_SVC);
+		SG_INFO("created SVMlibsvm object for multiclass\n") ;
+	}
+	else if (strcmp(name,"SCATTERSVM")==0)
+	{
+		SG_UNREF(classifier);
+		classifier= new CScatterSVM();
+		SG_INFO("created ScatterSVM object\n") ;
+	}
+	else if (strcmp(name,"LIBSVM_NU")==0)
+	{
+		SG_UNREF(classifier);
+		classifier= new CLibSVM(LIBSVM_NU_SVC);
+		SG_INFO("created SVMlibsvm object\n") ;
+	}
 	else if (strcmp(name,"LIBSVM")==0)
 	{
 		SG_UNREF(classifier);
 		classifier= new CLibSVM();
 		SG_INFO("created SVMlibsvm object\n") ;
 	}
+	else if (strcmp(name,"LARANK")==0)
+	{
+		SG_UNREF(classifier);
+		classifier= new CLaRank();
+		SG_INFO("created LaRank object\n") ;
+	}
 
 	else if (strcmp(name,"GPBTSVM")==0)
 	{
@@ -310,6 +344,31 @@ bool CGUIClassifier::new_classifier(char* name, int32_t d, int32_t from_d)
 		((CSVMOcas*) classifier)->set_bias_enabled(svm_use_bias);
 		SG_INFO("created SVM Ocas(BMRM/PERF) object\n") ;
 	}
+	else if (strcmp(name,"MKL_CLASSIFICATION")==0)
+	{
+		SG_UNREF(classifier);
+		classifier= new CMKLClassification();
+	}
+	//else if (strcmp(name,"MKL_MULTICLASS")==0)
+	//{
+	//	SG_UNREF(classifier);
+	//	classifier= new CMKLClassification();
+	//}
+	else if (strcmp(name,"MKL_ONECLASS")==0)
+	{
+		SG_UNREF(classifier);
+		classifier= new CMKLOneClass();
+	}
+	else if (strcmp(name,"MKL_MULTICLASS")==0)
+	{
+		SG_UNREF(classifier);
+		classifier= new CMKLMultiClass();
+	}
+	else if (strcmp(name,"MKL_REGRESSION")==0)
+	{
+		SG_UNREF(classifier);
+		classifier= new CMKLRegression();
+	}
 	else
 	{
 		SG_ERROR("Unknown classifier %s.\n", name);
@@ -317,11 +376,121 @@ bool CGUIClassifier::new_classifier(char* name, int32_t d, int32_t from_d)
 	}
 	SG_REF(classifier);
 
-	classifier->set_solver_type(solver_type);
-
 	return (classifier!=NULL);
 }
 
+bool CGUIClassifier::train_mkl_multiclass()
+{
+	CMKLMultiClass* mkl= (CMKLMultiClass*) classifier;
+	if (!mkl)
+		SG_ERROR("No MKL available.\n");
+
+	CLabels* trainlabels=ui->ui_labels->get_train_labels();
+	if (!trainlabels)
+		SG_ERROR("No trainlabels available.\n");
+
+	CKernel* kernel=ui->ui_kernel->get_kernel();
+	if (!kernel)
+		SG_ERROR("No kernel available.\n");
+
+	bool success=ui->ui_kernel->init_kernel("TRAIN");
+
+	if (!success || !ui->ui_kernel->is_initialized() || !kernel->has_features())
+		SG_ERROR("Kernel not initialized / no train features available.\n");
+
+	int32_t num_vec=kernel->get_num_vec_lhs();
+	if (trainlabels->get_num_labels() != num_vec)
+		SG_ERROR("Number of train labels (%d) and training vectors (%d) differs!\n", trainlabels->get_num_labels(), num_vec);
+
+	SG_INFO("Starting MC-MKL training on %ld vectors using C1=%lf C2=%lf epsilon=%lf\n", num_vec, svm_C1, svm_C2, svm_epsilon);
+
+	mkl->set_mkl_epsilon(svm_weight_epsilon);
+	//mkl->set_max_num_mkliters(-1);
+	mkl->set_solver_type(solver_type);
+	mkl->set_bias_enabled(svm_use_bias);
+	mkl->set_epsilon(svm_epsilon);
+	mkl->set_max_train_time(max_train_time);
+	mkl->set_tube_epsilon(svm_tube_epsilon);
+	mkl->set_nu(svm_nu);
+	mkl->set_C(svm_C1, svm_C2);
+	mkl->set_qpsize(svm_qpsize);
+	mkl->set_shrinking_enabled(svm_use_shrinking);
+	mkl->set_linadd_enabled(svm_use_linadd);
+	mkl->set_batch_computation_enabled(svm_use_batch_computation);
+
+	((CKernelMachine*) mkl)->set_labels(trainlabels);
+	((CKernelMachine*) mkl)->set_kernel(kernel);
+
+	return mkl->train();
+}
+
+bool CGUIClassifier::train_mkl()
+{
+	CMKL* mkl= (CMKL*) classifier;
+	if (!mkl)
+		SG_ERROR("No SVM available.\n");
+
+	bool oneclass=(mkl->get_classifier_type()==CT_LIBSVMONECLASS);
+	CLabels* trainlabels=NULL;
+	if(!oneclass)
+		trainlabels=ui->ui_labels->get_train_labels();
+	else
+		SG_INFO("Training one class mkl.\n");
+	if (!trainlabels && !oneclass)
+		SG_ERROR("No trainlabels available.\n");
+
+	CKernel* kernel=ui->ui_kernel->get_kernel();
+	if (!kernel)
+		SG_ERROR("No kernel available.\n");
+
+	bool success=ui->ui_kernel->init_kernel("TRAIN");
+	if (!success || !ui->ui_kernel->is_initialized() || !kernel->has_features())
+		SG_ERROR("Kernel not initialized.\n");
+
+	int32_t num_vec=kernel->get_num_vec_lhs();
+	if (!oneclass && trainlabels->get_num_labels() != num_vec)
+		SG_ERROR("Number of train labels (%d) and training vectors (%d) differs!\n", trainlabels->get_num_labels(), num_vec);
+
+	SG_INFO("Starting SVM training on %ld vectors using C1=%lf C2=%lf epsilon=%lf\n", num_vec, svm_C1, svm_C2, svm_epsilon);
+
+	if (constraint_generator)
+		mkl->set_constraint_generator(constraint_generator);
+	mkl->set_solver_type(solver_type);
+	mkl->set_bias_enabled(svm_use_bias);
+	mkl->set_epsilon(svm_epsilon);
+	mkl->set_max_train_time(max_train_time);
+	mkl->set_tube_epsilon(svm_tube_epsilon);
+	mkl->set_nu(svm_nu);
+	mkl->set_C(svm_C1, svm_C2);
+	mkl->set_qpsize(svm_qpsize);
+	mkl->set_shrinking_enabled(svm_use_shrinking);
+	mkl->set_linadd_enabled(svm_use_linadd);
+	mkl->set_batch_computation_enabled(svm_use_batch_computation);
+	mkl->set_mkl_epsilon(svm_weight_epsilon);
+	mkl->set_mkl_norm(mkl_norm); 
+	mkl->set_C_mkl(C_mkl);
+	mkl->set_interleaved_optimization_enabled(mkl_use_interleaved);
+
+	if (svm_do_auc_maximization)
+	{
+		CAUCKernel* auc_kernel = new CAUCKernel(10, kernel);
+		CLabels* auc_labels= auc_kernel->setup_auc_maximization(trainlabels);
+		((CKernelMachine*) mkl)->set_labels(auc_labels);
+		((CKernelMachine*) mkl)->set_kernel(auc_kernel);
+		SG_UNREF(auc_labels);
+	}
+	else
+	{
+		if(!oneclass)
+			((CKernelMachine*) mkl)->set_labels(trainlabels);
+		((CKernelMachine*) mkl)->set_kernel(kernel);
+	}
+
+	bool result=mkl->train();
+
+	return result;
+}
+
 bool CGUIClassifier::train_svm()
 {
 	CSVM* svm= (CSVM*) classifier;
@@ -340,8 +509,11 @@ bool CGUIClassifier::train_svm()
 	CKernel* kernel=ui->ui_kernel->get_kernel();
 	if (!kernel)
 		SG_ERROR("No kernel available.\n");
-	if (!ui->ui_kernel->is_initialized() || !kernel->has_features())
-		SG_ERROR("Kernel not initialized.\n");
+
+	bool success=ui->ui_kernel->init_kernel("TRAIN");
+
+	if (!success || !ui->ui_kernel->is_initialized() || !kernel->has_features())
+		SG_ERROR("Kernel not initialized / no train features available.\n");
 
 	int32_t num_vec=kernel->get_num_vec_lhs();
 	if (!oneclass && trainlabels->get_num_labels() != num_vec)
@@ -349,21 +521,23 @@ bool CGUIClassifier::train_svm()
 
 	SG_INFO("Starting SVM training on %ld vectors using C1=%lf C2=%lf epsilon=%lf\n", num_vec, svm_C1, svm_C2, svm_epsilon);
 
+	svm->set_solver_type(solver_type);
 	svm->set_bias_enabled(svm_use_bias);
-	svm->set_weight_epsilon(svm_weight_epsilon);
-	svm->set_mkl_norm(svm_mkl_norm);
 	svm->set_epsilon(svm_epsilon);
 	svm->set_max_train_time(max_train_time);
 	svm->set_tube_epsilon(svm_tube_epsilon);
 	svm->set_nu(svm_nu);
-	svm->set_C_mkl(svm_C_mkl);
 	svm->set_C(svm_C1, svm_C2);
 	svm->set_qpsize(svm_qpsize);
-	svm->set_mkl_enabled(svm_use_mkl);
 	svm->set_shrinking_enabled(svm_use_shrinking);
 	svm->set_linadd_enabled(svm_use_linadd);
 	svm->set_batch_computation_enabled(svm_use_batch_computation);
 
+	if(svm->get_classifier_type()==CT_MKLMULTICLASS)
+	{
+		((CMKLMultiClass *)svm)->set_mkl_epsilon(svm_weight_epsilon ); 
+	}
+
 	if (svm_do_auc_maximization)
 	{
 		CAUCKernel* auc_kernel = new CAUCKernel(10, kernel);
@@ -392,6 +566,9 @@ bool CGUIClassifier::train_clustering(int32_t k, int32_t max_iter)
 	if (!distance)
 		SG_ERROR("No distance available\n");
 
+	if (!ui->ui_distance->init_distance("TRAIN"))
+		SG_ERROR("Initializing distance with train features failed.\n");
+
 	((CDistanceMachine*) classifier)->set_distance(distance);
 
 	EClassifierType type=classifier->get_classifier_type();
@@ -428,6 +605,8 @@ bool CGUIClassifier::train_knn(int32_t k)
 	{
 		if (distance)
 		{
+			if (!ui->ui_distance->init_distance("TRAIN"))
+				SG_ERROR("Initializing distance with train features failed.\n");
 			((CKNN*) classifier)->set_labels(trainlabels);
 			((CKNN*) classifier)->set_distance(distance);
 			((CKNN*) classifier)->set_k(k);
@@ -442,6 +621,38 @@ bool CGUIClassifier::train_knn(int32_t k)
 	return result;
 }
 
+bool CGUIClassifier::train_krr()
+{
+#ifdef HAVE_LAPACK
+	CKRR* krr= (CKRR*) classifier;
+	if (!krr)
+		SG_ERROR("No SVM available.\n");
+
+	CLabels* trainlabels=NULL;
+	trainlabels=ui->ui_labels->get_train_labels();
+	if (!trainlabels)
+		SG_ERROR("No trainlabels available.\n");
+
+	CKernel* kernel=ui->ui_kernel->get_kernel();
+	if (!kernel)
+		SG_ERROR("No kernel available.\n");
+
+	bool success=ui->ui_kernel->init_kernel("TRAIN");
+
+	if (!success || !ui->ui_kernel->is_initialized() || !kernel->has_features())
+		SG_ERROR("Kernel not initialized / no train features available.\n");
+
+	int32_t num_vec=kernel->get_num_vec_lhs();
+	if (trainlabels->get_num_labels() != num_vec)
+		SG_ERROR("Number of train labels (%d) and training vectors (%d) differs!\n", trainlabels->get_num_labels(), num_vec);
+
+	bool result=krr->train();
+	return result;
+#else
+	return false;
+#endif
+}
+
 bool CGUIClassifier::train_linear(float64_t gamma)
 {
 	ASSERT(classifier);
@@ -691,8 +902,12 @@ bool CGUIClassifier::set_max_train_time(float64_t max)
 
 bool CGUIClassifier::set_svr_tube_epsilon(float64_t tube_epsilon)
 {
+	if (!classifier)
+		SG_ERROR("No regression method allocated\n");
+
 	if (classifier->get_classifier_type() != CT_LIBSVR &&
-			classifier->get_classifier_type() != CT_SVRLIGHT)
+			classifier->get_classifier_type() != CT_SVRLIGHT &&
+			classifier->get_classifier_type() != CT_MKLREGRESSION )
 	{
 		SG_ERROR("Underlying method not capable of SV-regression\n");
 	}
@@ -707,32 +922,34 @@ bool CGUIClassifier::set_svr_tube_epsilon(float64_t tube_epsilon)
 	return true;
 }
 
-bool CGUIClassifier::set_svm_one_class_nu(float64_t nu)
+bool CGUIClassifier::set_svm_nu(float64_t nu)
 {
 	if (nu<0 || nu>1)
-		svm_nu=0.5;
+		nu=0.5;
+
+	svm_nu=nu;
 	SG_INFO("Set to nu=%f.\n", svm_nu);
 
 	return true;
 }
 
 bool CGUIClassifier::set_svm_mkl_parameters(
-	float64_t weight_epsilon, float64_t C_mkl, float64_t mkl_norm)
+	float64_t weight_epsilon, float64_t C, float64_t norm)
 {
 	if (weight_epsilon<0)
-		svm_weight_epsilon=1e-4;
-	if (C_mkl<0)
-		svm_C_mkl=0;
-	if (mkl_norm<=0)
+		weight_epsilon=1e-4;
+	if (C<0)
+		C=0;
+	if (norm<=0)
 		SG_ERROR("MKL norm > 0\n");
 
 	svm_weight_epsilon=weight_epsilon;
-	svm_C_mkl=C_mkl;
-	svm_mkl_norm=mkl_norm;
+	C_mkl=C;
+	mkl_norm=norm;
 
 	SG_INFO("Set to weight_epsilon=%f.\n", svm_weight_epsilon);
-	SG_INFO("Set to C_mkl=%f.\n", svm_C_mkl);
-	SG_INFO("Set to mkl_norm=%f.\n", svm_mkl_norm);
+	SG_INFO("Set to C_mkl=%f.\n", C_mkl);
+	SG_INFO("Set to mkl_norm=%f.\n", mkl_norm);
 
 	return true;
 }
@@ -787,17 +1004,6 @@ bool CGUIClassifier::set_svm_bufsize(int32_t bufsize)
 	return true ;
 }
 
-bool CGUIClassifier::set_svm_mkl_enabled(bool enabled)
-{
-	svm_use_mkl=enabled;
-	if (svm_use_mkl)
-		SG_INFO("Enabling MKL optimization.\n");
-	else
-		SG_INFO("Disabling MKL optimization.\n");
-
-	return true;
-}
-
 bool CGUIClassifier::set_svm_shrinking_enabled(bool enabled)
 {
 	svm_use_shrinking=enabled;
@@ -842,6 +1048,17 @@ bool CGUIClassifier::set_svm_bias_enabled(bool enabled)
 	return true;
 }
 
+bool CGUIClassifier::set_mkl_interleaved_enabled(bool enabled)
+{
+	mkl_use_interleaved=enabled;
+	if (mkl_use_interleaved)
+		SG_INFO("Enabling mkl interleaved optimization.\n");
+	else
+		SG_INFO("Disabling mkl interleaved optimization.\n");
+
+	return true;
+}
+
 bool CGUIClassifier::set_do_auc_maximization(bool do_auc)
 {
 	svm_do_auc_maximization=do_auc;
@@ -855,7 +1072,7 @@ bool CGUIClassifier::set_do_auc_maximization(bool do_auc)
 }
 
 
-CLabels* CGUIClassifier::classify(CLabels* output)
+CLabels* CGUIClassifier::classify()
 {
 	ASSERT(classifier);
 
@@ -863,6 +1080,7 @@ CLabels* CGUIClassifier::classify(CLabels* output)
 	{
 		case CT_LIGHT:
 		case CT_LIBSVM:
+		case CT_SCATTERSVM:
 		case CT_MPD:
 		case CT_GPBT:
 		case CT_CPLEXSVM:
@@ -873,13 +1091,17 @@ CLabels* CGUIClassifier::classify(CLabels* output)
 		case CT_LIBSVMMULTICLASS:
 		case CT_LIBSVMONECLASS:
 		case CT_SVRLIGHT:
+		case CT_MKLCLASSIFICATION:
+		case CT_MKLMULTICLASS:
+		case CT_MKLREGRESSION:
+		case CT_MKLONECLASS:
 		case CT_KRR:
-			return classify_kernelmachine(output);
+			return classify_kernelmachine();
 		case CT_KNN:
-			return classify_distancemachine(output);
+			return classify_distancemachine();
 		case CT_PERCEPTRON:
 		case CT_LDA:
-			return classify_linear(output);
+			return classify_linear();
 		case CT_SVMLIN:
 		case CT_SVMPERF:
 		case CT_SUBGRADIENTSVM:
@@ -889,9 +1111,9 @@ CLabels* CGUIClassifier::classify(CLabels* output)
 		case CT_LPBOOST:
 		case CT_SUBGRADIENTLPM:
 		case CT_LIBLINEAR:
-			return classify_linear(output);
+			return classify_linear();
 		case CT_WDSVMOCAS:
-			return classify_byte_linear(output);
+			return classify_byte_linear();
 		default:
 			SG_ERROR("unknown classifier type\n");
 			break;
@@ -900,7 +1122,7 @@ CLabels* CGUIClassifier::classify(CLabels* output)
 	return false;
 }
 
-CLabels* CGUIClassifier::classify_kernelmachine(CLabels* output)
+CLabels* CGUIClassifier::classify_kernelmachine()
 {
 	CFeatures* trainfeatures=ui->ui_features->get_train_features();
 	CFeatures* testfeatures=ui->ui_features->get_test_features();
@@ -908,15 +1130,28 @@ CLabels* CGUIClassifier::classify_kernelmachine(CLabels* output)
 	if (!classifier)
 		SG_ERROR("No kernelmachine available.\n");
 
-	if (!ui->ui_kernel->get_kernel() ||
-			!ui->ui_kernel->get_kernel()->get_kernel_type()==K_CUSTOM)
+	bool success=true;
+
+	if (ui->ui_kernel->get_kernel()->get_kernel_type()!=K_CUSTOM)
 	{
-		if (!trainfeatures)
-			SG_ERROR("No training features available.\n");
-		if (!testfeatures)
-			SG_ERROR("No test features available.\n");
+		if (ui->ui_kernel->get_kernel()->get_kernel_type()==K_COMBINED
+				&& ( !trainfeatures || !testfeatures ))
+		{
+			SG_DEBUG("skipping initialisation of combined kernel "
+					"as train/test features are unavailable\n");
+		}
+		else
+		{
+			if (!trainfeatures)
+				SG_ERROR("No training features available.\n");
+			if (!testfeatures)
+				SG_ERROR("No test features available.\n");
+
+			success=ui->ui_kernel->init_kernel("TEST");
+		}
 	}
-	if (!ui->ui_kernel->is_initialized())
+
+	if (!success || !ui->ui_kernel->is_initialized())
 		SG_ERROR("Kernel not initialized.\n");
 
 	CKernelMachine* km=(CKernelMachine*) classifier;
@@ -924,7 +1159,7 @@ CLabels* CGUIClassifier::classify_kernelmachine(CLabels* output)
 	km->set_batch_computation_enabled(svm_use_batch_computation);
 
 	SG_INFO("Starting kernel machine testing.\n");
-	return classifier->classify(output);
+	return classifier->classify();
 }
 
 bool CGUIClassifier::get_trained_classifier(
@@ -936,18 +1171,23 @@ bool CGUIClassifier::get_trained_classifier(
 
 	switch (classifier->get_classifier_type())
 	{
+		case CT_SCATTERSVM:
+		case CT_GNPPSVM:
+		case CT_LIBSVMMULTICLASS:
 		case CT_LIGHT:
 		case CT_LIBSVM:
 		case CT_MPD:
 		case CT_GPBT:
 		case CT_CPLEXSVM:
 		case CT_GMNPSVM:
-		case CT_GNPPSVM:
 		case CT_KERNELPERCEPTRON:
 		case CT_LIBSVR:
-		case CT_LIBSVMMULTICLASS:
 		case CT_LIBSVMONECLASS:
 		case CT_SVRLIGHT:
+		case CT_MKLCLASSIFICATION:
+		case CT_MKLREGRESSION:
+		case CT_MKLONECLASS:
+		case CT_MKLMULTICLASS:
 		case CT_KRR:
 			return get_svm(weights, rows, cols, bias, brows, bcols, idx);
 			break;
@@ -985,7 +1225,6 @@ int32_t CGUIClassifier::get_num_svms()
 	return ((CMultiClassSVM*) classifier)->get_num_svms();
 }
 
-
 bool CGUIClassifier::get_svm(
 	float64_t* &weights, int32_t& rows, int32_t& cols, float64_t*& bias,
 	int32_t& brows, int32_t& bcols, int32_t idx)
@@ -1096,7 +1335,7 @@ bool CGUIClassifier::get_linear(
 	return true;
 }
 
-CLabels* CGUIClassifier::classify_distancemachine(CLabels* output)
+CLabels* CGUIClassifier::classify_distancemachine()
 {
 	CFeatures* trainfeatures=ui->ui_features->get_train_features();
 	CFeatures* testfeatures=ui->ui_features->get_test_features();
@@ -1118,7 +1357,9 @@ CLabels* CGUIClassifier::classify_distancemachine(CLabels* output)
 		return NULL;
 	}
 
-	if (!ui->ui_distance->is_initialized())
+	bool success=ui->ui_distance->init_distance("TEST");
+
+	if (!success || !ui->ui_distance->is_initialized())
 	{
 		SG_ERROR("distance not initialized\n") ;
 		return NULL;
@@ -1127,11 +1368,11 @@ CLabels* CGUIClassifier::classify_distancemachine(CLabels* output)
 	((CDistanceMachine*) classifier)->set_distance(
 		ui->ui_distance->get_distance());
 	SG_INFO("starting distance machine testing\n") ;
-	return classifier->classify(output);
+	return classifier->classify();
 }
 
 
-CLabels* CGUIClassifier::classify_linear(CLabels* output)
+CLabels* CGUIClassifier::classify_linear()
 {
 	CFeatures* testfeatures=ui->ui_features->get_test_features();
 
@@ -1153,10 +1394,10 @@ CLabels* CGUIClassifier::classify_linear(CLabels* output)
 
 	((CLinearClassifier*) classifier)->set_features((CDotFeatures*) testfeatures);
 	SG_INFO("starting linear classifier testing\n") ;
-	return classifier->classify(output);
+	return classifier->classify();
 }
 
-CLabels* CGUIClassifier::classify_byte_linear(CLabels* output)
+CLabels* CGUIClassifier::classify_byte_linear()
 {
 	CFeatures* testfeatures=ui->ui_features->get_test_features();
 
@@ -1179,7 +1420,7 @@ CLabels* CGUIClassifier::classify_byte_linear(CLabels* output)
 
 	((CWDSVMOcas*) classifier)->set_features((CStringFeatures<uint8_t>*) testfeatures);
 	SG_INFO("starting linear classifier testing\n") ;
-	return classifier->classify(output);
+	return classifier->classify();
 }
 
 bool CGUIClassifier::classify_example(int32_t idx, float64_t &result)
@@ -1240,10 +1481,15 @@ bool CGUIClassifier::set_solver(char* solver)
 {
 	ESolverType s=ST_AUTO;
 
-	if (strncmp(solver,"INTERNAL", 8)==0)
+	if (strncmp(solver,"NEWTON", 6)==0)
 	{
-		SG_INFO("Using INTERNAL solver.\n");
-		s=ST_INTERNAL;
+		SG_INFO("Using NEWTON solver.\n");
+		s=ST_NEWTON;
+	}
+	else if (strncmp(solver,"DIRECT", 6)==0)
+	{
+		SG_INFO("Using DIRECT solver\n");
+		s=ST_DIRECT;
 	}
 	else if (strncmp(solver,"AUTO", 4)==0)
 	{
@@ -1271,3 +1517,88 @@ bool CGUIClassifier::set_solver(char* solver)
 	solver_type=s;
 	return true;
 }
+
+bool CGUIClassifier::set_constraint_generator(char* name)
+{
+	if (strcmp(name,"LIBSVM_ONECLASS")==0)
+	{
+		SG_UNREF(constraint_generator);
+		constraint_generator = new CLibSVMOneClass();
+		SG_INFO("created SVMlibsvm object for oneclass\n");
+	}
+	else if (strcmp(name,"LIBSVM_MULTICLASS")==0)
+	{
+		SG_UNREF(constraint_generator);
+		constraint_generator = new CLibSVMMultiClass();
+		SG_INFO("created SVMlibsvm object for multiclass\n");
+	}
+	else if (strcmp(name,"LIBSVM_NUMULTICLASS")==0)
+	{
+		SG_UNREF(constraint_generator);
+		constraint_generator= new CLibSVMMultiClass(LIBSVM_NU_SVC);
+		SG_INFO("created SVMlibsvm object for multiclass\n") ;
+	}
+	else if (strcmp(name,"SCATTERSVM")==0)
+	{
+		SG_UNREF(constraint_generator);
+		constraint_generator= new CScatterSVM();
+		SG_INFO("created ScatterSVM object\n") ;
+	}
+	else if (strcmp(name,"LIBSVM_NU")==0)
+	{
+		SG_UNREF(constraint_generator);
+		constraint_generator= new CLibSVM(LIBSVM_NU_SVC);
+		SG_INFO("created SVMlibsvm object\n") ;
+	}
+	else if (strcmp(name,"LIBSVM")==0)
+	{
+		SG_UNREF(constraint_generator);
+		constraint_generator= new CLibSVM();
+		SG_INFO("created SVMlibsvm object\n") ;
+	}
+	else if (strcmp(name,"LARANK")==0)
+	{
+		SG_UNREF(constraint_generator);
+		constraint_generator= new CLaRank();
+		SG_INFO("created LaRank object\n") ;
+	}
+
+	else if (strcmp(name,"GPBTSVM")==0)
+	{
+		SG_UNREF(constraint_generator);
+		constraint_generator= new CGPBTSVM();
+		SG_INFO("created GPBT-SVM object\n") ;
+	}
+	else if (strcmp(name,"MPDSVM")==0)
+	{
+		SG_UNREF(constraint_generator);
+		constraint_generator= new CMPDSVM();
+		SG_INFO("created MPD-SVM object\n") ;
+	}
+	else if (strcmp(name,"GNPPSVM")==0)
+	{
+		SG_UNREF(constraint_generator);
+		constraint_generator= new CGNPPSVM();
+		SG_INFO("created GNPP-SVM object\n") ;
+	}
+	else if (strcmp(name,"GMNPSVM")==0)
+	{
+		SG_UNREF(constraint_generator);
+		constraint_generator= new CGMNPSVM();
+		SG_INFO("created GMNP-SVM object\n") ;
+	}
+	else if (strcmp(name,"LIBSVR")==0)
+	{
+		SG_UNREF(constraint_generator);
+		constraint_generator= new CLibSVR();
+		SG_INFO("created SVRlibsvm object\n") ;
+	}
+	else
+	{
+		SG_ERROR("Unknown SV-classifier %s.\n", name);
+		return false;
+	}
+	SG_REF(constraint_generator);
+
+	return (constraint_generator!=NULL);
+}
