@@ -46,23 +46,25 @@
 #include <shogun/kernel/SparsePolyKernel.h>
 #include <shogun/kernel/SparseGaussianKernel.h>
 #include <shogun/kernel/DiagKernel.h>
-#include <shogun/kernel/MindyGramKernel.h>
 #include <shogun/kernel/OligoStringKernel.h>
 #include <shogun/kernel/DistanceKernel.h>
 #include <shogun/kernel/TensorProductPairKernel.h>
 #include <shogun/kernel/AvgDiagKernelNormalizer.h>
+#include <shogun/kernel/RidgeKernelNormalizer.h>
 #include <shogun/kernel/FirstElementKernelNormalizer.h>
 #include <shogun/kernel/IdentityKernelNormalizer.h>
 #include <shogun/kernel/SqrtDiagKernelNormalizer.h>
+#include <shogun/kernel/VarianceKernelNormalizer.h>
 #include <shogun/classifier/svm/SVM.h>
 
 #include <string.h>
 
+using namespace shogun;
+
 CGUIKernel::CGUIKernel(CSGInterface* ui_)
 : CSGObject(), ui(ui_)
 {
 	kernel=NULL;
-	initialized=false;
 }
 
 CGUIKernel::~CGUIKernel()
@@ -75,25 +77,6 @@ CKernel* CGUIKernel::get_kernel()
 	return kernel;
 }
 
-#ifdef HAVE_MINDY
-CKernel* CGUIKernel::create_mindygram(
-	int32_t size, char* meas_str, char* norm_str, float64_t width,
-	char* param_str)
-{
-	CKernel* kern=new CMindyGramKernel(size, meast_str, width);
-	if (!kern)
-		SG_ERROR("Couldn't create MindyGramKernel with size %d, meas_str %s, width %f.\n", size, meas_str, width);
-	else
-		SG_DEBUG("created MindyGramKernel (%p) with size %d, meas_str %s, width %f.\n", kern, size, meas_str, width);
-
-	ENormalizationType normalization=get_normalization_from_str(norm_str);
-	kern->set_norm(normalization);
-	kern->set_param(param_str);
-
-	return kern;
-}
-#endif
-
 CKernel* CGUIKernel::create_oligo(int32_t size, int32_t k, float64_t width)
 {
 	CKernel* kern=new COligoStringKernel(size, k, width);
@@ -624,7 +607,7 @@ CKernel* CGUIKernel::create_combined(
 	return kern;
 }
 
-bool CGUIKernel::set_normalization(char* normalization, float64_t c)
+bool CGUIKernel::set_normalization(char* normalization, float64_t c, float64_t r)
 {
 	CKernel* k=kernel;
 
@@ -644,6 +627,11 @@ bool CGUIKernel::set_normalization(char* normalization, float64_t c)
 		SG_INFO("Average Kernel Diagonal Normalization selected\n");
 		return k->set_normalizer(new CAvgDiagKernelNormalizer(c));
 	}
+	else if (strncmp(normalization,"RIDGE", 5)==0)
+	{
+		SG_INFO("Ridge Kernel Normalization selected\n");
+		return k->set_normalizer(new CRidgeKernelNormalizer(r, c));
+	}
 	else if (strncmp(normalization,"SQRTDIAG", 8)==0)
 	{
 		SG_INFO("Sqrt Diagonal Normalization selected\n");
@@ -654,6 +642,11 @@ bool CGUIKernel::set_normalization(char* normalization, float64_t c)
 		SG_INFO("First Element Normalization selected\n");
 		return k->set_normalizer(new CFirstElementKernelNormalizer());
 	}
+	else if (strncmp(normalization,"VARIANCE", 8)==0)
+	{
+		SG_INFO("Variance Normalization selected\n");
+		return k->set_normalizer(new CVarianceKernelNormalizer());
+	}
 	else
 		SG_ERROR("Wrong kernel normalizer name.\n");
 
@@ -670,11 +663,6 @@ bool CGUIKernel::set_kernel(CKernel* kern)
 		SG_UNREF(kernel);
 		SG_REF(kern);
 		kernel=kern;
-
-		// custom kernel does not require init / features
-		if (kern->get_kernel_type()==K_CUSTOM)
-			initialized=true;
-
 		SG_DEBUG("set new kernel (%p).\n", kern);
 
 		return true;
@@ -683,54 +671,6 @@ bool CGUIKernel::set_kernel(CKernel* kern)
 		return false;
 }
 
-bool CGUIKernel::load_kernel_init(char* filename)
-{
-	bool result=false;
-	if (kernel)
-	{
-		FILE* file=fopen(filename, "r");
-		if (!file || !kernel->load_init(file))
-			SG_ERROR("Reading from file %s failed!\n", filename);
-		else
-		{
-			SG_INFO("Successfully read kernel init data from %s!\n", filename);
-			initialized=true;
-			result=true;
-		}
-
-		if (file)
-			fclose(file);
-	}
-	else
-		SG_ERROR("No kernel set!\n");
-
-	return result;
-}
-
-bool CGUIKernel::save_kernel_init(char* filename)
-{
-	bool result=false;
-
-	if (kernel)
-	{
-		FILE* file=fopen(filename, "w");
-		if (!file || !kernel->save_init(file))
-			SG_ERROR("Writing to file %s failed!\n", filename);
-		else
-		{
-			SG_INFO("Successfully written kernel init data into %s!\n", filename);
-			result=true;
-		}
-
-		if (file)
-			fclose(file);
-	}
-	else
-		SG_ERROR("No kernel set!\n");
-
-	return result;
-}
-
 bool CGUIKernel::init_kernel_optimization()
 {
 	CSVM* svm=(CSVM*) ui->ui_classifier->get_classifier();
@@ -773,11 +713,15 @@ bool CGUIKernel::delete_kernel_optimization()
 }
 
 
-bool CGUIKernel::init_kernel(char* target)
+bool CGUIKernel::init_kernel(const char* target)
 {
 	if (!kernel)
 		SG_ERROR("No kernel available.\n");
 
+	// no need to init custom kernel
+	if (kernel->get_kernel_type() == K_CUSTOM || !target)
+		return true;
+
 	EFeatureClass k_fclass=kernel->get_feature_class();
 	EFeatureType k_ftype=kernel->get_feature_type();
 
@@ -800,7 +744,7 @@ bool CGUIKernel::init_kernel(char* target)
 				SG_ERROR("Kernel can not process this train feature type: %d %d.\n", fclass, ftype);
 		}
 		else
-			SG_ERROR("Assign train features first.\n");
+			SG_DEBUG("Not initing kernel - no train features assigned.\n");
 	}
 	else if (!strncmp(target, "TEST", 4))
 	{
@@ -827,7 +771,7 @@ bool CGUIKernel::init_kernel(char* target)
 				SG_ERROR("Kernel can not process this test feature type: %d %d.\n", fclass, ftype);
 		}
 		else
-			SG_ERROR("Assign train and test features first.\n");
+			SG_DEBUG("Not initing kernel - no train and test features assigned.\n");
 	}
 	else
 		SG_ERROR("Unknown target %s.\n", target);
@@ -866,11 +810,10 @@ bool CGUIKernel::add_kernel(CKernel* kern, float64_t weight)
 
 	if (kernel->get_kernel_type()!=K_COMBINED)
 	{
-		CKernel* first_elem=kern;
+		CKernel* first_elem=kernel;
 		kernel= new CCombinedKernel(20, false);
 		SG_REF(kernel);
 		((CCombinedKernel*) kernel)->append_kernel(first_elem);
-		SG_UNREF(first_elem);
 	}
 
 	if (!kernel)
