@@ -1,9 +1,11 @@
 #ifndef __SGINTERFACE__H_
 #define __SGINTERFACE__H_
 
+#include <shogun/lib/memory.h>
 #include <shogun/lib/config.h>
 #include <shogun/lib/common.h>
 #include <shogun/base/SGObject.h>
+#include <shogun/features/Features.h>
 #include <shogun/features/StringFeatures.h>
 #include <shogun/features/SparseFeatures.h>
 #include <shogun/features/AttributeFeatures.h>
@@ -21,6 +23,8 @@
 #include "GUITime.h"
 #include "GUIStructure.h"
 
+namespace shogun
+{
 /* Interface datatypes that shogun supports. Note that some interfaces like e.g.
  * octave/matlab cannot distinguish between scalars and matrices and thus might
  * always return more complex types like matrices.
@@ -81,6 +85,16 @@ enum IFType
 	ATTR_STRUCT
 };
 
+enum E_WHICH_OBJ
+{
+	SVM_PRIMAL,
+	SVM_DUAL,
+	MKL_PRIMAL,
+	MKL_DUAL,
+	MKL_RELATIVE_DUALITY_GAP,
+	MKL_ABSOLUTE_DUALITY_GAP
+};
+
 class CSGInterface : public CSGObject
 {
 	public:
@@ -104,6 +118,8 @@ class CSGInterface : public CSGObject
 		bool cmd_get_features();
 		/** add features */
 		bool cmd_add_features();
+		/** add multiple features */
+		bool cmd_add_multiple_features();
 		/** add dot features */
 		bool cmd_add_dotfeatures();
 		/** set features */
@@ -114,10 +130,6 @@ class CSGInterface : public CSGObject
 		bool cmd_del_last_features();
 		/** convert features */
 		bool cmd_convert();
-		/** obtain from position list */
-		bool cmd_obtain_from_position_list();
-		/** obtain by sliding window */
-		bool cmd_obtain_by_sliding_window();
 		/** reshape features */
 		bool cmd_reshape();
 		/** load labels from file */
@@ -179,6 +191,8 @@ class CSGInterface : public CSGObject
 		bool cmd_set_kernel_optimization_type();
 		/** set solver type */
 		bool cmd_set_solver();
+		/** set constraint generator */
+		bool cmd_set_constraint_generator();
 		/** set Salzberg prior probs */
 		bool cmd_set_prior_probs();
 		/** set Salzberg prior probs from labels */
@@ -216,6 +230,8 @@ class CSGInterface : public CSGObject
 		bool cmd_get_num_svms();
 		/** set SVM */
 		bool cmd_set_svm();
+		/** set linear classifier */
+		bool cmd_set_linear_classifier();
 		/** classify */
 		bool cmd_classify();
 		/** classify example */
@@ -224,6 +240,16 @@ class CSGInterface : public CSGObject
 		bool cmd_get_classifier();
 		/** get SVM objective */
 		bool cmd_get_svm_objective();
+		/** compute SVM objective from scratch*/
+		bool cmd_compute_svm_primal_objective();
+		/** compute SVM objective from scratch*/
+		bool cmd_compute_svm_dual_objective();
+		/** compute SVM objective from scratch*/
+		bool cmd_compute_mkl_dual_objective();
+		/** compute relative mkl duality gap */
+		bool cmd_compute_relative_mkl_duality_gap();
+		/** compute absolute mkl duality gap */
+		bool cmd_compute_absolute_mkl_duality_gap();
 		/** train classifier/SVM */
 		bool cmd_train_classifier();
 		/** test SVM */
@@ -245,7 +271,7 @@ class CSGInterface : public CSGObject
 		/** set SVR tube epsilon */
 		bool cmd_set_svr_tube_epsilon();
 		/** set SVM OneClass nu */
-		bool cmd_set_svm_one_class_nu();
+		bool cmd_set_svm_nu();
 		/** set SVM MKL parameters */
 		bool cmd_set_svm_mkl_parameters();
 		/** set max train time */
@@ -260,6 +286,8 @@ class CSGInterface : public CSGObject
 		bool cmd_set_svm_linadd_enabled();
 		/** set SVM bias enabled */
 		bool cmd_set_svm_bias_enabled();
+		/** set MKL intebias enabled */
+		bool cmd_set_mkl_interleaved_enabled();
 		/** set krr tau */
 		bool cmd_set_krr_tau();
 
@@ -267,10 +295,6 @@ class CSGInterface : public CSGObject
 		bool cmd_add_preproc();
 		/** delete preproc */
 		bool cmd_del_preproc();
-		/** load preproc from file */
-		bool cmd_load_preproc();
-		/** save preproc to file */
-		bool cmd_save_preproc();
 		/** attach preproc to test/train */
 		bool cmd_attach_preproc();
 		/** clear/clean preproc */
@@ -411,6 +435,11 @@ class CSGInterface : public CSGObject
 		 */
 		bool cmd_init_dyn_prog();
 
+		bool cmd_init_intron_list();
+
+		/** settings for long transition approximation*/
+		bool cmd_long_transition_settings();
+
 		/**
 		 * -precompute tiling features 
 		 *  and save the outputs (# content types x 
@@ -432,6 +461,11 @@ class CSGInterface : public CSGObject
  		 *   content features and tiling features
  		 */
 		bool cmd_set_model();
+		/**
+		 * set sparse feature matrix and 
+		 * all feature positions
+		 */	
+		bool cmd_set_feature_matrix_sparse();
 		/**
 		 * set feature matrix and 
 		 * all feature positions
@@ -443,8 +477,6 @@ class CSGInterface : public CSGObject
 		bool cmd_best_path_trans_deriv();
 		/** best path no b */
 		bool cmd_best_path_no_b();
-		/** best path trans simple */
-		bool cmd_best_path_trans_simple();
 		/** best path no b trans */
 		bool cmd_best_path_no_b_trans();
 
@@ -486,6 +518,8 @@ class CSGInterface : public CSGObject
 		bool cmd_get_version();
 		/** issue help message */
 		bool cmd_help();
+		/** list alloc'd memory */
+		bool cmd_whos();
 		/** wrapper for compatibility send_command */
 		bool cmd_send_command();
 		/** execute code under python from octave,... */
@@ -707,17 +741,30 @@ class CSGInterface : public CSGObject
 			return get_string(len);
 		}
 	private:
+		/** helper function for computing objective */
+		bool do_compute_objective(E_WHICH_OBJ obj);
 		/** helper function for hmm classify */
 		bool do_hmm_classify(bool linear=false, bool one_class=false);
 		/** helper function for hmm classify on 1 example */
 		bool do_hmm_classify_example(bool one_class=false);
 		/** helper function for add/set features */
-		bool do_set_features(bool add=false, bool check_dot=false);
+		bool do_set_features(bool add=false, bool check_dot=false, int32_t repetitions=1);
+
+		/** perform bit embedding */
+		void convert_to_bitembedding(CFeatures* &features, bool convert_to_word, bool convert_to_ulong);
+		/** obtain from single string */
+		void obtain_from_single_string(CFeatures* features);
+		/** obtain from position list */
+		bool obtain_from_position_list(CFeatures* features);
+		/** obtain by sliding window */
+		bool obtain_by_sliding_window(CFeatures* features);
 		/** helper function to create a kernel */
 		CKernel* create_kernel();
 
 		/** helper function to create certain string features */
 		CFeatures* create_custom_string_features(CStringFeatures<uint8_t>* f);
+
+		CFeatures* create_custom_real_features(CSimpleFeatures<float64_t>* orig_feat);
 		/** legacy-related stuff - anybody got a better idea? */
 		char* get_str_from_str_or_direct(int32_t& len);
 		int32_t get_int_from_int_or_str();
@@ -757,5 +804,6 @@ typedef struct {
 	const char* usage_prefix;
 	const char* usage_suffix;
 } CSGInterfaceMethod;
+}
 
 #endif // __SGINTERFACE__H_
