@@ -23,7 +23,7 @@
 #include <shogun/ui/SGInterface.h>
 #include <shogun/lib/ShogunException.h>
 #include <shogun/lib/io.h>
-#include <shogun/lib/memory.h>
+#include <shogun/lib/config.h>
 #include <shogun/base/init.h>
 
 #ifdef HAVE_PYTHON
@@ -35,6 +35,8 @@
 #undef length
 #endif
 
+#include "../libshogun/lib/memory.cpp"
+
 void octave_print_message(FILE* target, const char* str)
 {
 	fprintf(target, "%s", str);
@@ -282,7 +284,7 @@ void COctaveInterface::function_name(sg_type*& matrix, int32_t& num_feat, int32_
 	oct_type m = mat_feat.oct_converter();												\
 	num_vec = m.cols();																\
 	num_feat = m.rows();															\
-	matrix=new sg_type[num_vec*num_feat];											\
+	matrix = new sg_type[num_vec*num_feat];											\
 																					\
 	for (int32_t i=0; i<num_vec; i++)													\
 		for (int32_t j=0; j<num_feat; j++)												\
@@ -600,11 +602,30 @@ void COctaveInterface::recover_from_exception(void)
   can_interrupt = true;
   octave_interrupt_immediately = 0;
   octave_interrupt_state = 0;
+
+#if OCTAVE_APIVERSION >= 37
+  octave_exception_state = octave_no_exception;
+#else
   octave_allocation_error = 0;
+#endif
   octave_restore_signal_mask ();
   octave_catch_interrupts ();
 }
 
+void COctaveInterface::clear_octave_globals()
+{
+	//string_vector gvars = symbol_table::global_variable_names();
+
+	//int gcount = gvars.length();
+
+	//for (int i = 0; i < gcount; i++)
+	//	symbol_table::clear_global(gvars[i]);
+	int parse_status;
+	eval_string("clear all", false, parse_status);
+	//	symbol_table::clear_global_pattern ("*");
+	//global_sym_tab->clear();
+}
+
 void COctaveInterface::run_octave_init()
 {
 	char* name=strdup("octave");
@@ -644,7 +665,7 @@ bool COctaveInterface::run_octave_helper(CSGInterface* from_if)
 	{
 		int parse_status;
 		char* octave_code=NULL;
-		global_sym_tab->clear();
+		clear_octave_globals();
 
 		for (int i=0; i<from_if->get_nrhs(); i++)
 		{
@@ -665,14 +686,21 @@ bool COctaveInterface::run_octave_helper(CSGInterface* from_if)
 				COctaveInterface* in = new COctaveInterface(args, 1, false);
 				in->create_return_values(1);
 				from_if->translate_arg(from_if, in);
+#if OCTAVE_APIVERSION >= 37
+				symbol_table::varref (var_name) = in->get_return_values()(0);
+#else
 				set_global_value(var_name, in->get_return_values()(0));
+#endif
 				delete[] var_name;
 				SG_UNREF(in);
 			}
 		}
 
+#if OCTAVE_APIVERSION >= 37
+#else
 		symbol_table* old=curr_sym_tab;
 		curr_sym_tab = global_sym_tab;
+#endif
 		reset_error_handler ();
 		eval_string(octave_code, false, parse_status);
 		delete[] octave_code;
@@ -680,11 +708,20 @@ bool COctaveInterface::run_octave_helper(CSGInterface* from_if)
 		int32_t sz=0;
 		octave_value_list results;
 
+#if OCTAVE_APIVERSION >= 37
+		if (symbol_table::is_variable("results"))
+		{
+			results = symbol_table::varval("results");
+			//results = get_global_value("results", false);
+			sz=results.length();
+		}
+#else
 		if (curr_sym_tab->lookup("results"))
 		{
 			results = get_global_value("results", false);
 			sz=results.length();
 		}
+#endif
 
 		if (sz>0)
 		{
@@ -696,7 +733,6 @@ bool COctaveInterface::run_octave_helper(CSGInterface* from_if)
 			}
 		}
 
-
 		if (sz>0 && from_if->create_return_values(sz))
 		{
 			from_if->SG_DEBUG("Found %d args\n", sz);
@@ -717,17 +753,20 @@ bool COctaveInterface::run_octave_helper(CSGInterface* from_if)
 			}
 		}
 
+#if OCTAVE_APIVERSION >= 37
+#else
 		curr_sym_tab=old;
+#endif
 	}
 	catch (octave_interrupt_exception)
 	{
 		recover_from_exception ();
-		std::cout << "\n"; 
+		SG_SPRINT("%\n");
 	}
 	catch (std::bad_alloc)
 	{
 		recover_from_exception ();
-		std::cout << "\n"; 
+		SG_SPRINT("%\n");
 	}
 
 	octave_restore_signal_mask();
